/* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause
 *
 * Copyright (c) 2020-2021 Senscomm Semiconductor Co., Ltd.
 *
 */
#ifndef __SCM80211_COEX_H__
#define __SCM80211_COEX_H__

#include <linux/types.h>
#include <linux/workqueue.h>
#include <linux/slab.h>
#include <linux/regmap.h>
#include <linux/proc_fs.h>
#include <linux/platform_device.h>
#include <linux/pci.h>
#include <linux/usb.h>
#include <linux/average.h>
#include <linux/hrtimer.h>
#include <linux/version.h>

#include <net/mac80211.h>
#include <linux/version.h>

#include "cfg80211.h"
#include "hrtimer.h"
#include "skbuff.h"
#include "ieee80211ax.h"
#include "hw.h"
#include "phy.h"
#include "random.h"
#include "debug.h"

#ifdef SCM80211_COEX

#define SCM80211_COEX_TIMEOUT 					10	/* 10ms */
#define SCM80211_COEX_2ANT_MONITOR_PERIOD_200	200 /* 200ms */
#define SCM80211_COEX_2ANT_MONITOR_PERIOD_1000	1000 /* 1s */

union htc_omi {
	__le16 content;
	struct {
		__le16	id		:bits(3,  0);
		__le16	rx_nss		:bits(6,  4);
		__le16	bw		:bits(8,  7);
		__le16	ul_mudis	:bits(9,  9);
		__le16	tx_nss		:bits(12, 10);
		__le16	ersu_dis	:bits(13, 13);
		__le16	dl_mu_rec	:bits(14, 14);
		__le16	ul_mudatadis	:bits(15, 15);
	};
};

struct scm80211_coex {
	u8 dev_ant;
	bool coex_en;
	bool reduce_streams_flag;
	bool reduce_streams_bypass;
	int bt_action_counter;
	union htc_omi htc_omi;
	struct ieee80211_hw *hw;
	struct ieee80211_sta *sta;
	struct hrtimer stop_queue_watchdog;
	struct hrtimer coex_2ant_timer;
};

void scm80211_enable_coex_irq(struct ieee80211_hw *hw);
void scm80211_disable_coex_irq(struct ieee80211_hw *hw);
void scm80211_coex_irq(struct ieee80211_hw *hw);
void scm80211_set_coex(struct ieee80211_hw *hw, struct scm80211_chanctx *ctx);
enum hrtimer_restart scm80211_coex_2ant_mon_timer(struct hrtimer *timer);
enum hrtimer_restart scm80211_hwq_stop_queue_watchdog_expire(struct hrtimer *timer);
void scm80211_pta_pri_set(struct ieee80211_hw *hw, u8 wlan);

#endif
#endif /* __SCM80211_COEX_H__ */
