/* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause */
/*
 * Copyright (c) 2020-2023 Senscomm Semiconductor Co., Ltd.
 *
 */

#ifndef __SCM80211_DEBUG_H__
#define __SCM80211_DEBUG_H__

#include <net/mac80211.h>

enum scm80211_debug_index {
	MAIN_DBG	= BIT(0),
	TX_DBG		= BIT(1),
	RX_DBG		= BIT(2),
	CAL_DBG		= BIT(3),
	BUS_DBG		= BIT(4),
	RC_DBG		= BIT(5),
	CHAN_DBG	= BIT(6),
	PHY_DBG		= BIT(7),
	RF_DBG		= BIT(8),
	PS_DBG		= BIT(9),
	DFS_DBG		= BIT(10),
	FS_DBG		= BIT(11),
	FTM_DBG		= BIT(12),
	COEX_DBG 	= BIT(13),
	PRIV_DBG	= BIT(15),
};

#ifdef func_trace
#define trace(func)	_trace(func)
#define _trace(func)	func ## _DBG
#else
#define trace(func)	0
#endif

#define scm80211_trace(hw, format, args...)	\
{ \
	if (ieee80211_hw_to_priv(hw)->debug_trace & trace(func_trace)) { \
		wiphy_dbg((hw)->wiphy, func_fmt(format), ##args); \
	} \
} \

#define scm80211_dbg(hw, format, args...) \
	wiphy_dbg((hw)->wiphy, func_fmt(format), ##args)

#define scm80211_info(hw, format, args...) \
	wiphy_info((hw)->wiphy, func_fmt(format), ##args)

#define scm80211_warn(hw, format, args...) \
	wiphy_warn((hw)->wiphy, func_fmt(format), ##args)

#define scm80211_err(hw, format, args...) \
	wiphy_err((hw)->wiphy, func_fmt(format), ##args)

#endif /* __SCM80211_DEBUG_H__ */
