/* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause */
/*
 * Copyright (c) 2020-2023 Senscomm Semiconductor Co., Ltd.
 */
#ifndef __EFUSE_H__
#define	__EFUSE_H__
#include <linux/types.h>
#include "core.h"
#include "util.h"

#ifdef EFUSE_ENABLE

#define EFUSE_WR_SIZE			24
#define EFUSE_BANK_SIZE		32
#define EFUSE_USED_SIZE		14

enum sens_efuse_banks {
//      EFUSE_BANK_0 = 0,
	EFUSE_BANK_1 = 1,
	EFUSE_BANK_2,
	EFUSE_MAX_BANK = 2,
};

#define MAX_EFUSE_SIZE			(EFUSE_MAX_BANK * EFUSE_BANK_SIZE)

enum sens_efuse_field_ids {
	SENS_EFUSE_FLD_ID_INVALID = 0,
	SENS_EFUSE_XCAL_FLD_ID,
	SENS_EFUSE_XXD_FLD_ID,

	SENS_EFUSE_WIFI_MAC_FLD_ID,
	SENS_EFUSE_BT_MAC_FLD_ID,

	SENS_EFUSE_TEMP_FLD_ID_CAL_2G,
	SENS_EFUSE_TEMP_FLD_ID_CAL_5G,

	SENS_EFUSE_RESV_FLD_ID_7,
	SENS_EFUSE_RESV_FLD_ID_8,

	SENS_EFUSE_TXPWR_FLD_ID_CAL_2G,
	SENS_EFUSE_TXPWR_FLD_ID_CAL_5G,
	SENS_EFUSE_BOTH_TXPWR_FLD_ID_CAL,

	SENS_EFUSE_RESV_FLD_ID_12,

	SENS_EFUSE_RSSI_FLD_ID_CAL_2G,
	SENS_EFUSE_RSSI_FLD_ID_CAL_5G,
	SENS_EFUSE_BOTH_RSSI_FLD_ID_CAL,

	MAX_EFUSE_FLD_ID,	// 16
};

struct efuse_temper_type {
	u8 tx_temp_sloop_2g;
	u8 rx_temp_sloop_2g;
	u8 tx_temp_sloop_5g;
	u8 rx_temp_sloop_5g;
};

struct efuse_xx_flag_type {
	u8 t_t_d:1;
	u8 t_r_d:1;
	u8 f_t_d:1;
	u8 f_r_d:1;
	u8 A_n:1;
	u8 b_d:1;
	u8 b_u:1;
	u8 S_0:1;
};

struct efuse_rssi_pwr_2g {
	u8 ch0_rpwr_2g:4;
	u8 ch1_rpwr_2g:4;

	u8 ch0_rpwr_2g_unt:1;
	u8 ch1_rpwr_2g_unt:1;
	u8 rpwr_2g_unused:6;
};

struct efuse_rssi_pwr_5g {
	u8 ch0_rpwr_5g_l:4;
	u8 ch1_rpwr_5g_l:4;
	u8 ch0_rpwr_5g_m:4;
	u8 ch1_rpwr_5g_m:4;
	u8 ch0_rpwr_5g_h:4;
	u8 ch1_rpwr_5g_h:4;

	u8 ch0_rpwr_5g_l_unt:1;
	u8 ch1_rpwr_5g_l_unt:1;
	u8 ch0_rpwr_5g_m_unt:1;
	u8 ch1_rpwr_5g_m_unt:1;
	u8 ch0_rpwr_5g_h_unt:1;
	u8 ch1_rpwr_5g_h_unt:1;
	u8 rpwr_5g_unused:2;
};

struct efuse_pwr_0_5_step_type {
	u8 ch0_rpwr_2g_unt:1;
	u8 ch1_rpwr_2g_unt:1;

	u8 ch0_rpwr_5g_l_unt:1;
	u8 ch1_rpwr_5g_l_unt:1;
	u8 ch0_rpwr_5g_m_unt:1;
	u8 ch1_rpwr_5g_m_unt:1;
	u8 ch0_rpwr_5g_h_unt:1;
	u8 ch1_rpwr_5g_h_unt:1;
};

struct efuse_rssi_pwr_type {
	u8 ch0_rpwr_2g:4;
	u8 ch1_rpwr_2g:4;

	u8 ch0_rpwr_5g_l:4;
	u8 ch1_rpwr_5g_l:4;
	u8 ch0_rpwr_5g_m:4;
	u8 ch1_rpwr_5g_m:4;
	u8 ch0_rpwr_5g_h:4;
	u8 ch1_rpwr_5g_h:4;
};

struct efuse_tag_bits {
	u8 efuse_tag_bl:4;
	u8 efuse_tag_bh:4;
};

union efuse_tag_type {
	u8 efuse_tag_x;
	struct efuse_tag_bits tag_b;
};

struct efuse_tag_mac_type {
	u8 efuse_wifi_mac_L[3];
	union efuse_tag_type efuse_tag_t0;
	u8 efuse_wifi_mac_H[3];
	union efuse_tag_type efuse_tag_t1;
	u8 efuse_bt_mac_L[3];
	union efuse_tag_type efuse_tag_t2;
	u8 efuse_bt_mac_H[3];
};

struct efuse_tag_id_type {
	u8 efuse_ver:4;
	u8 rf_cal_ver:4;
};

struct efuse_resv2_map {
	u8 xtal_cal;
	u8 resv_2[3];
};

struct efuse_resv_map {
	struct efuse_resv2_map resv_2;
	u8 resv_tags[EFUSE_WR_SIZE - 4];
};

struct efuse_bit_map {
	u8 efuse_used_bytes[EFUSE_USED_SIZE];

	struct efuse_tag_id_type efuse_id_t;

	struct efuse_xx_flag_type flag_t;

	struct efuse_pwr_0_5_step_type efuse_pwr_step;

	struct efuse_tag_mac_type efuse_tmac_t;

	//      0x00    BK2
	struct efuse_rssi_pwr_type efuse_rpwr_t;

	struct efuse_temper_type efuse_temp_t;

	u8 resv_next[EFUSE_WR_SIZE];
} __packed;

// read data type API
struct efuse_bdata_map {
	u32 efuse_ver;
	u32 rf_cal_ver;

	u32 xtal_cal;
	u32 t_t_d;
	u32 t_r_d;
	u32 f_t_d;
	u32 f_r_d;
	u32 ant_sel;
	u32 bt_dn_flag;
	u32 bt_wdis_uart;
	u32 sens_oui;

	u32 tx_temp_2g_cal;
	u32 rx_temp_2g_cal;
	u32 tx_temp_5g_cal;
	u32 rx_temp_5g_cal;

	// power = [7:4]*10 + [3:0]
	u32 ch0_pwr_2g_cal;
	u32 ch1_pwr_2g_cal;
	u32 ch0_pwr_5gl_cal;
	u32 ch1_pwr_5gl_cal;
	u32 ch0_pwr_5gm_cal;
	u32 ch1_pwr_5gm_cal;
	u32 ch0_pwr_5gh_cal;
	u32 ch1_pwr_5gh_cal;

	u32 ch0_rssi_2g_cal;
	u32 ch1_rssi_2g_cal;
	u32 ch0_rssi_5gl_cal;
	u32 ch1_rssi_5gl_cal;
	u32 ch0_rssi_5gm_cal;
	u32 ch1_rssi_5gm_cal;
	u32 ch0_rssi_5gh_cal;
	u32 ch1_rssi_5gh_cal;

	union bd_mac_type {
		u32 bd_mac_val[4];
		struct wb_mac_type {
			u8 wmac_cflag;
			u8 wmac_resv;
			u8 wifi_mac[6];

			u8 bmac_cflag;
			u8 bmac_resv;
			u8 bt_mac[6];
		} wb_mac_t;
	} bd_mac_t;

	u32 bk2_wsize;
	u32 end_flag;		//51*4 Byte
};

#define	BD_MAP_SIZE		sizeof(struct efuse_bdata_map)


int scm80211_efuse_set_cmd_proc(char *cmd_str, char *rsp);
int scm80211_efuse_get_cmd_proc(char *cmd_str, char *rsp);
int scm80211_efuse_pcie_init(void *ep);
int scm80211_efuse_cal_update(u8 path, int *cal);

bool scm80211_efuse_mac_update(u8 *wmac, u8 *btmac);
bool scm80211_efuse_usb_init(void *udev, void *buff, int length);
int scm80211_efuse_get_ant_sel(void);
int scm80211_efuse_get_xtal_cal(void);


#endif /* EFUSE_ENABLE */
#endif /* #define      __EFUSE__ */
