/* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause */
/*
 * Copyright (c) 2020-2023 Senscomm Semiconductor Co., Ltd.
 *
 */
#ifndef __SCM80211_REGS_H__
#define __SCM80211_REGS_H__

#include "scm2020/v0.9.8/macregs_32fef74d_FPGA_v0.9.7.h"
#include "scm2020/v0.9.8/phyregs_scm2020_B0BA01AC.h"

/*
 * MAC register map
 */

#define MAC_IMEM_OFFSET			0x0000
#define MAC_DMEM_OFFSET			0x4000
#define MAC_SHM_OFFSET			0x4760
#define MAC_FIFO_OFFSET			0x6000
#define MAC_MISC_OFFSET			0x7000
#define MAC_REG_OFFSET			0x8000
#define MAC_SYS_OFFSET			0x9000
#define MAC_TIM_OFFSET			0x9100
#define MAC_TXE_OFFSET			0x9300
#define MAC_RXE_OFFSET			0x9400

/*
 * MAC shared memory region
 */

#define MAC_SHM_TXDESC(x)		(MAC_SHM_OFFSET + ((x) * 0x10))
#define TXDESC_MDESC_ADDR_LO(x)		(MAC_SHM_TXDESC(x) + 0x0)
#define TXDESC_MDESC_ADDR_HI(x)		(MAC_SHM_TXDESC(x) + 0x4)
#define TXDESC_NUM_MPDU_RESULT(x)	(MAC_SHM_TXDESC(x) + 0x8)
#define	TXDESC_NUM_MPDU			GENMASK(5, 0)
#define TXDESC_TX_HTC(x)		(MAC_SHM_TXDESC(x) + 0xc)

#define MAC_SHM_RXDESC(x)		(MAC_SHM_OFFSET + 0xa0 + ((x) * 0x10))
#define RXDESC_BUF_ADDR_LO(x)		(MAC_SHM_RXDESC(x) + 0x0)
#define RXDESC_BUF_ADDR_HI(x)		(MAC_SHM_RXDESC(x) + 0x4)
#define RXDESC_BUF_SIZE(x)		(MAC_SHM_RXDESC(x) + 0x8)
#define MAC_MISC_NULL_HEADER		(MAC_MISC_OFFSET + 0x13C)
#define TIM_IFS_CFG1			(MAC_TIM_OFFSET + 0x14)
#define TXE_TIM_INFO0			(MAC_TXE_OFFSET + 0x10)

/*
 * MAC register definitions
 * #define MAC_REG_INTR_ENABLE	(MAC_REG_OFFSET + REG_INTR_ENABLE)
 */

#define MAC_REG_INTR_ENABLE			(MAC_REG_OFFSET + REG_INTR_ENABLE)
#define MAC_REG_INTR_CLEAR			(MAC_REG_OFFSET + REG_INTR_CLEAR)
#define MAC_REG_INTR_STATUS			(MAC_REG_OFFSET + REG_INTR_STATUS)
#define MAC_REG_MAC_CFG				(MAC_REG_OFFSET + REG_MAC_CFG)
#define MAC_REG_MAC_STATUS			(MAC_REG_OFFSET + REG_MAC_STATUS)
#define MAC_REG_MCU_CFG				(MAC_REG_OFFSET + REG_MCU_CFG)
#define MAC_REG_MCU_STATUS			(MAC_REG_OFFSET + REG_MCU_STATUS)
#define MAC_REG_VER_INFO			(MAC_REG_OFFSET + REG_VER_INFO)
#define MAC_REG_GIT_VER				(MAC_REG_OFFSET + REG_GIT_VER)
#define MAC_REG_DEV_CFG				(MAC_REG_OFFSET + REG_DEV_CFG)
#define MAC_REG_DEV_OPT1			(MAC_REG_OFFSET + REG_DEV_OPT1)
#define MAC_REG_DEV_CS				(MAC_REG_OFFSET + REG_DEV_CS)
#define MAC_REG_DEV_CH_COEX			(MAC_REG_OFFSET + REG_DEV_CH_COEX)
#define MAC_REG_DEV_PMU				(MAC_REG_OFFSET + REG_DEV_PMU)
#define MAC_REG_DEV_FORCE_CLR			(MAC_REG_OFFSET + REG_DEV_FORCE_CLR)
#define MAC_REG_DEV_OPT2			(MAC_REG_OFFSET + REG_DEV_OPT2)
#define MAC_REG_DEV_FORCE_QUE_CLR		(MAC_REG_OFFSET + REG_DEV_FORCE_QUE_CLR)
#define MAC_REG_DEV_CLK_CFG			(MAC_REG_OFFSET + REG_DEV_CLK_CFG)
#define MAC_REG_INTR_SELECT			(MAC_REG_OFFSET + REG_INTR_SELECT)
#define MAC_REG_VIF0_CFG0			(MAC_REG_OFFSET + REG_VIF0_CFG0)
#define MAC_REG_VIF0_CFG1			(MAC_REG_OFFSET + REG_VIF0_CFG1)
#define MAC_REG_VIF0_RX_FILTER0			(MAC_REG_OFFSET + REG_VIF0_RX_FILTER0)
#define MAC_REG_VIF0_RX_FILTER1			(MAC_REG_OFFSET + REG_VIF0_RX_FILTER1)
#define MAC_REG_VIF0_MAC0_L			(MAC_REG_OFFSET + REG_VIF0_MAC0_L)
#define MAC_REG_VIF0_MAC0_H			(MAC_REG_OFFSET + REG_VIF0_MAC0_H)
#define MAC_REG_VIF0_MAC1_L			(MAC_REG_OFFSET + REG_VIF0_MAC1_L)
#define MAC_REG_VIF0_MAC1_H			(MAC_REG_OFFSET + REG_VIF0_MAC1_H)
#define MAC_REG_VIF0_BSS0_L			(MAC_REG_OFFSET + REG_VIF0_BSS0_L)
#define MAC_REG_VIF0_BSS0_H			(MAC_REG_OFFSET + REG_VIF0_BSS0_H)
#define MAC_REG_VIF0_BSS1_L			(MAC_REG_OFFSET + REG_VIF0_BSS1_L)
#define MAC_REG_VIF0_BSS1_H			(MAC_REG_OFFSET + REG_VIF0_BSS1_H)
#define MAC_REG_VIF0_AID			(MAC_REG_OFFSET + REG_VIF0_AID)
#define MAC_REG_VIF0_BFM0			(MAC_REG_OFFSET + REG_VIF0_BFM0)
#define MAC_REG_VIF0_BFM1			(MAC_REG_OFFSET + REG_VIF0_BFM1)
#define MAC_REG_VIF0_BFM2			(MAC_REG_OFFSET + REG_VIF0_BFM2)
#define MAC_REG_VIF0_TSF_L			(MAC_REG_OFFSET + REG_VIF0_TSF_L)
#define MAC_REG_VIF0_TSF_H			(MAC_REG_OFFSET + REG_VIF0_TSF_H)
#define MAC_REG_VIF0_TSF_OFFSET_L		(MAC_REG_OFFSET + REG_VIF0_TSF_OFFSET_L)
#define MAC_REG_VIF0_TSF_OFFSET_H		(MAC_REG_OFFSET + REG_VIF0_TSF_OFFSET_H)
#define MAC_REG_VIF0_TSF_TIME0			(MAC_REG_OFFSET + REG_VIF0_TSF_TIME0)
#define MAC_REG_VIF0_TSF_TIME1			(MAC_REG_OFFSET + REG_VIF0_TSF_TIME1)
#define MAC_REG_VIF0_TRIG_RESP_QUEUE_EN		(MAC_REG_OFFSET + REG_VIF0_TRIG_RESP_QUEUE_EN)
#define MAC_REG_VIF0_BSR			(MAC_REG_OFFSET + REG_VIF0_BSR)
#define MAC_REG_VIF0_MIN_MPDU_SPACING		(MAC_REG_OFFSET + REG_VIF0_MIN_MPDU_SPACING)
#define MAC_REG_VIF0_QNULL_CTRL_AC0		(MAC_REG_OFFSET + REG_VIF0_QNULL_CTRL_AC0)
#define MAC_REG_VIF0_QNULL_CTRL_AC1		(MAC_REG_OFFSET + REG_VIF0_QNULL_CTRL_AC1)
#define MAC_REG_VIF0_QNULL_CTRL_AC2		(MAC_REG_OFFSET + REG_VIF0_QNULL_CTRL_AC2)
#define MAC_REG_VIF0_QNULL_CTRL_AC3		(MAC_REG_OFFSET + REG_VIF0_QNULL_CTRL_AC3)
#define MAC_REG_VIF0_UORA_CFG			(MAC_REG_OFFSET + REG_VIF0_UORA_CFG)
#define MAC_REG_VIF0_UORA_CFG_STATUS		(MAC_REG_OFFSET + REG_VIF0_UORA_CFG_STATUS)
#define MAC_REG_VIF0_UORA_CFG1			(MAC_REG_OFFSET + REG_VIF0_UORA_CFG1)
#define MAC_REG_VIF0_NON_SRG_OBSS_PD		(MAC_REG_OFFSET + REG_VIF0_NON_SRG_OBSS_PD)
#define MAC_REG_VIF0_SRG_OBSS_PD		(MAC_REG_OFFSET + REG_VIF0_SRG_OBSS_PD)
#define MAC_REG_VIF0_TX_PWR			(MAC_REG_OFFSET + REG_VIF0_TX_PWR)
#define MAC_REG_VIF0_CFG_INFO			(MAC_REG_OFFSET + REG_VIF0_CFG_INFO)
#define MAC_REG_VIF0_MBSSID_MASK_L32		(MAC_REG_OFFSET + REG_VIF0_MBSSID_MASK_L32)
#define MAC_REG_VIF0_MBSSID_MASK_H16		(MAC_REG_OFFSET + REG_VIF0_MBSSID_MASK_H16)
#define MAC_REG_VIF0_TRIG			(MAC_REG_OFFSET + REG_VIF0_TRIG)
#define MAC_REG_VIF0_ESP			(MAC_REG_OFFSET + REG_VIF0_ESP)
#define MAC_REG_VIF0_ESP_STATUS			(MAC_REG_OFFSET + REG_VIF0_ESP_STATUS)
#define MAC_REG_VIF0_HTC_OMI			(MAC_REG_OFFSET + REG_VIF0_HTC_OMI)
#define MAC_REG_VIF0_BASIC_TRIG_USER_INFO	(MAC_REG_OFFSET + REG_VIF0_BASIC_TRIG_USER_INFO)
#define MAC_REG_VIF0_BASIC_TRIG_PSDU_LEN	(MAC_REG_OFFSET + REG_VIF0_BASIC_TRIG_PSDU_LEN)
#define MAC_REG_VIF0_RSVD			(MAC_REG_OFFSET + REG_VIF0_RSVD)
#define MAC_REG_VIF1_CFG0			(MAC_REG_OFFSET + REG_VIF1_CFG0)
#define MAC_REG_VIF1_CFG1			(MAC_REG_OFFSET + REG_VIF1_CFG1)
#define MAC_REG_VIF1_RX_FILTER0			(MAC_REG_OFFSET + REG_VIF1_RX_FILTER0)
#define MAC_REG_VIF1_RX_FILTER1			(MAC_REG_OFFSET + REG_VIF1_RX_FILTER1)
#define MAC_REG_VIF1_MAC0_L			(MAC_REG_OFFSET + REG_VIF1_MAC0_L)
#define MAC_REG_VIF1_MAC0_H			(MAC_REG_OFFSET + REG_VIF1_MAC0_H)
#define MAC_REG_VIF1_MAC1_L			(MAC_REG_OFFSET + REG_VIF1_MAC1_L)
#define MAC_REG_VIF1_MAC1_H			(MAC_REG_OFFSET + REG_VIF1_MAC1_H)
#define MAC_REG_VIF1_BSS0_L			(MAC_REG_OFFSET + REG_VIF1_BSS0_L)
#define MAC_REG_VIF1_BSS0_H			(MAC_REG_OFFSET + REG_VIF1_BSS0_H)
#define MAC_REG_VIF1_BSS1_L			(MAC_REG_OFFSET + REG_VIF1_BSS1_L)
#define MAC_REG_VIF1_BSS1_H			(MAC_REG_OFFSET + REG_VIF1_BSS1_H)
#define MAC_REG_VIF1_AID			(MAC_REG_OFFSET + REG_VIF1_AID)
#define MAC_REG_VIF1_BFM0			(MAC_REG_OFFSET + REG_VIF1_BFM0)
#define MAC_REG_VIF1_BFM1			(MAC_REG_OFFSET + REG_VIF1_BFM1)
#define MAC_REG_VIF1_BFM2			(MAC_REG_OFFSET + REG_VIF1_BFM2)
#define MAC_REG_VIF1_TSF_L			(MAC_REG_OFFSET + REG_VIF1_TSF_L)
#define MAC_REG_VIF1_TSF_H			(MAC_REG_OFFSET + REG_VIF1_TSF_H)
#define MAC_REG_VIF1_TSF_OFFSET_L		(MAC_REG_OFFSET + REG_VIF1_TSF_OFFSET_L)
#define MAC_REG_VIF1_TSF_OFFSET_H		(MAC_REG_OFFSET + REG_VIF1_TSF_OFFSET_H)
#define MAC_REG_VIF1_TSF_TIME0			(MAC_REG_OFFSET + REG_VIF1_TSF_TIME0)
#define MAC_REG_VIF1_TSF_TIME1			(MAC_REG_OFFSET + REG_VIF1_TSF_TIME1)
#define MAC_REG_VIF1_TRIG_RESP_QUEUE_EN		(MAC_REG_OFFSET + REG_VIF1_TRIG_RESP_QUEUE_EN)
#define MAC_REG_VIF1_BSR			(MAC_REG_OFFSET + REG_VIF1_BSR)
#define MAC_REG_VIF1_MIN_MPDU_SPACING		(MAC_REG_OFFSET + REG_VIF1_MIN_MPDU_SPACING)
#define MAC_REG_VIF1_QNULL_CTRL_AC0		(MAC_REG_OFFSET + REG_VIF1_QNULL_CTRL_AC0)
#define MAC_REG_VIF1_QNULL_CTRL_AC1		(MAC_REG_OFFSET + REG_VIF1_QNULL_CTRL_AC1)
#define MAC_REG_VIF1_QNULL_CTRL_AC2		(MAC_REG_OFFSET + REG_VIF1_QNULL_CTRL_AC2)
#define MAC_REG_VIF1_QNULL_CTRL_AC3		(MAC_REG_OFFSET + REG_VIF1_QNULL_CTRL_AC3)
#define MAC_REG_VIF1_UORA_CFG			(MAC_REG_OFFSET + REG_VIF1_UORA_CFG)
#define MAC_REG_VIF1_UORA_CFG_STATUS		(MAC_REG_OFFSET + REG_VIF1_UORA_CFG_STATUS)
#define MAC_REG_VIF1_UORA_CFG1			(MAC_REG_OFFSET + REG_VIF1_UORA_CFG1)
#define MAC_REG_VIF1_NON_SRG_OBSS_PD		(MAC_REG_OFFSET + REG_VIF1_NON_SRG_OBSS_PD)
#define MAC_REG_VIF1_SRG_OBSS_PD		(MAC_REG_OFFSET + REG_VIF1_SRG_OBSS_PD)
#define MAC_REG_VIF1_TX_PWR			(MAC_REG_OFFSET + REG_VIF1_TX_PWR)
#define MAC_REG_VIF1_CFG_INFO			(MAC_REG_OFFSET + REG_VIF1_CFG_INFO)
#define MAC_REG_VIF1_MBSSID_MASK_L32		(MAC_REG_OFFSET + REG_VIF1_MBSSID_MASK_L32)
#define MAC_REG_VIF1_MBSSID_MASK_H16		(MAC_REG_OFFSET + REG_VIF1_MBSSID_MASK_H16)
#define MAC_REG_VIF1_TRIG			(MAC_REG_OFFSET + REG_VIF1_TRIG)
#define MAC_REG_VIF1_ESP			(MAC_REG_OFFSET + REG_VIF1_ESP)
#define MAC_REG_VIF1_ESP_STATUS			(MAC_REG_OFFSET + REG_VIF1_ESP_STATUS)
#define MAC_REG_VIF1_HTC_OMI			(MAC_REG_OFFSET + REG_VIF1_HTC_OMI)
#define MAC_REG_VIF1_BASIC_TRIG_USER_INFO	(MAC_REG_OFFSET + REG_VIF1_BASIC_TRIG_USER_INFO)
#define MAC_REG_VIF1_BASIC_TRIG_PSDU_LEN	(MAC_REG_OFFSET + REG_VIF1_BASIC_TRIG_PSDU_LEN)
#define MAC_REG_VIF1_RSVD			(MAC_REG_OFFSET + REG_VIF1_RSVD)
#define MAC_REG_TX_QUEN_CMD0			(MAC_REG_OFFSET + REG_TX_QUEN_CMD0)
#define MAC_REG_TX_QUEN_TXOP0			(MAC_REG_OFFSET + REG_TX_QUEN_TXOP0)
#define MAC_REG_TX_QUEN_STATE0			(MAC_REG_OFFSET + REG_TX_QUEN_STATE0)
#define MAC_REG_TX_QUEN_TIMER0			(MAC_REG_OFFSET + REG_TX_QUEN_TIMER0)
#define MAC_REG_TX_QUEN_CMD1			(MAC_REG_OFFSET + REG_TX_QUEN_CMD1)
#define MAC_REG_TX_QUEN_TXOP1			(MAC_REG_OFFSET + REG_TX_QUEN_TXOP1)
#define MAC_REG_TX_QUEN_STATE1			(MAC_REG_OFFSET + REG_TX_QUEN_STATE1)
#define MAC_REG_TX_QUEN_TIMER1			(MAC_REG_OFFSET + REG_TX_QUEN_TIMER1)
#define MAC_REG_TX_QUEN_CMD2			(MAC_REG_OFFSET + REG_TX_QUEN_CMD2)
#define MAC_REG_TX_QUEN_TXOP2			(MAC_REG_OFFSET + REG_TX_QUEN_TXOP2)
#define MAC_REG_TX_QUEN_STATE2			(MAC_REG_OFFSET + REG_TX_QUEN_STATE2)
#define MAC_REG_TX_QUEN_TIMER2			(MAC_REG_OFFSET + REG_TX_QUEN_TIMER2)
#define MAC_REG_TX_QUEN_CMD3			(MAC_REG_OFFSET + REG_TX_QUEN_CMD3)
#define MAC_REG_TX_QUEN_TXOP3			(MAC_REG_OFFSET + REG_TX_QUEN_TXOP3)
#define MAC_REG_TX_QUEN_STATE3			(MAC_REG_OFFSET + REG_TX_QUEN_STATE3)
#define MAC_REG_TX_QUEN_TIMER3			(MAC_REG_OFFSET + REG_TX_QUEN_TIMER3)
#define MAC_REG_TX_QUEN_CMD4			(MAC_REG_OFFSET + REG_TX_QUEN_CMD4)
#define MAC_REG_TX_QUEN_TXOP4			(MAC_REG_OFFSET + REG_TX_QUEN_TXOP4)
#define MAC_REG_TX_QUEN_STATE4			(MAC_REG_OFFSET + REG_TX_QUEN_STATE4)
#define MAC_REG_TX_QUEN_TIMER4			(MAC_REG_OFFSET + REG_TX_QUEN_TIMER4)
#define MAC_REG_TX_QUEN_CMD5			(MAC_REG_OFFSET + REG_TX_QUEN_CMD5)
#define MAC_REG_TX_QUEN_TXOP5			(MAC_REG_OFFSET + REG_TX_QUEN_TXOP5)
#define MAC_REG_TX_QUEN_STATE5			(MAC_REG_OFFSET + REG_TX_QUEN_STATE5)
#define MAC_REG_TX_QUEN_TIMER5			(MAC_REG_OFFSET + REG_TX_QUEN_TIMER5)
#define MAC_REG_TX_QUEN_CMD6			(MAC_REG_OFFSET + REG_TX_QUEN_CMD6)
#define MAC_REG_TX_QUEN_TXOP6			(MAC_REG_OFFSET + REG_TX_QUEN_TXOP6)
#define MAC_REG_TX_QUEN_STATE6			(MAC_REG_OFFSET + REG_TX_QUEN_STATE6)
#define MAC_REG_TX_QUEN_TIMER6			(MAC_REG_OFFSET + REG_TX_QUEN_TIMER6)
#define MAC_REG_TX_QUEN_CMD7			(MAC_REG_OFFSET + REG_TX_QUEN_CMD7)
#define MAC_REG_TX_QUEN_TXOP7			(MAC_REG_OFFSET + REG_TX_QUEN_TXOP7)
#define MAC_REG_TX_QUEN_STATE7			(MAC_REG_OFFSET + REG_TX_QUEN_STATE7)
#define MAC_REG_TX_QUEN_TIMER7			(MAC_REG_OFFSET + REG_TX_QUEN_TIMER7)
#define MAC_REG_TX_QUEN_CMD8			(MAC_REG_OFFSET + REG_TX_QUEN_CMD8)
#define MAC_REG_TX_QUEN_TXOP8			(MAC_REG_OFFSET + REG_TX_QUEN_TXOP8)
#define MAC_REG_TX_QUEN_STATE8			(MAC_REG_OFFSET + REG_TX_QUEN_STATE8)
#define MAC_REG_TX_QUEN_TIMER8			(MAC_REG_OFFSET + REG_TX_QUEN_TIMER8)
#define MAC_REG_TX_QUEN_CMD9			(MAC_REG_OFFSET + REG_TX_QUEN_CMD9)
#define MAC_REG_TX_QUEN_TXOP9			(MAC_REG_OFFSET + REG_TX_QUEN_TXOP9)
#define MAC_REG_TX_QUEN_STATE9			(MAC_REG_OFFSET + REG_TX_QUEN_STATE9)
#define MAC_REG_TX_QUEN_TIMER9			(MAC_REG_OFFSET + REG_TX_QUEN_TIMER9)
#define MAC_REG_TX_TXTIME_CMD			(MAC_REG_OFFSET + REG_TX_TXTIME_CMD)
#define MAC_REG_TX_TXTIME_RESULT		(MAC_REG_OFFSET + REG_TX_TXTIME_RESULT)
#define MAC_REG_TX_TXTIME_TV0			(MAC_REG_OFFSET + REG_TX_TXTIME_TV0)
#define MAC_REG_TX_TXTIME_TV1			(MAC_REG_OFFSET + REG_TX_TXTIME_TV1)
#define MAC_REG_TX_TXTIME_TV2			(MAC_REG_OFFSET + REG_TX_TXTIME_TV2)
#define MAC_REG_TX_TXTIME_TV3			(MAC_REG_OFFSET + REG_TX_TXTIME_TV3)
#define MAC_REG_TX_TXTIME_TV4			(MAC_REG_OFFSET + REG_TX_TXTIME_TV4)
#define MAC_REG_TX_TXTIME_TV5			(MAC_REG_OFFSET + REG_TX_TXTIME_TV5)
#define MAC_REG_TX_TXTIME_TV6			(MAC_REG_OFFSET + REG_TX_TXTIME_TV6)
#define MAC_REG_TX_TO_CFG			(MAC_REG_OFFSET + REG_TX_TO_CFG)
#define MAC_REG_TX_TO_11B_CFG			(MAC_REG_OFFSET + REG_TX_TO_11B_CFG)
#define MAC_REG_TX_CFG				(MAC_REG_OFFSET + REG_TX_CFG)
#define MAC_REG_TX_HETB_QNULL			(MAC_REG_OFFSET + REG_TX_HETB_QNULL)
#define MAC_REG_RX_CFG				(MAC_REG_OFFSET + REG_RX_CFG)
#define MAC_REG_RX_BUF_CFG			(MAC_REG_OFFSET + REG_RX_BUF_CFG)
#define MAC_REG_RX_BUF_PTR0			(MAC_REG_OFFSET + REG_RX_BUF_PTR0)
#define MAC_REG_RX_BUF_PTR1			(MAC_REG_OFFSET + REG_RX_BUF_PTR1)
#define MAC_REG_RX_SCB_CTRL			(MAC_REG_OFFSET + REG_RX_SCB_CTRL)
#define MAC_REG_RX_TO_CFG			(MAC_REG_OFFSET + REG_RX_TO_CFG)
#define MAC_REG_RX_TO_11B_CFG			(MAC_REG_OFFSET + REG_RX_TO_11B_CFG)
#define MAC_REG_RX_DTIM_CFG			(MAC_REG_OFFSET + REG_RX_DTIM_CFG)
#define MAC_REG_RX_DTIM_INFO			(MAC_REG_OFFSET + REG_RX_DTIM_INFO)
#define MAC_REG_SEC_KEY_CMD			(MAC_REG_OFFSET + REG_SEC_KEY_CMD)
#define MAC_REG_SEC_KEY_STATUS			(MAC_REG_OFFSET + REG_SEC_KEY_STATUS)
#define MAC_REG_SEC_MAC_ADDR_L32		(MAC_REG_OFFSET + REG_SEC_MAC_ADDR_L32)
#define MAC_REG_SEC_MAC_ADDR_H16		(MAC_REG_OFFSET + REG_SEC_MAC_ADDR_H16)
#define MAC_REG_SEC_TEMP_KEY0			(MAC_REG_OFFSET + REG_SEC_TEMP_KEY0)
#define MAC_REG_SEC_TEMP_KEY1			(MAC_REG_OFFSET + REG_SEC_TEMP_KEY1)
#define MAC_REG_SEC_TEMP_KEY2			(MAC_REG_OFFSET + REG_SEC_TEMP_KEY2)
#define MAC_REG_SEC_TEMP_KEY3			(MAC_REG_OFFSET + REG_SEC_TEMP_KEY3)
#define MAC_REG_SEC_TEMP_KEY4			(MAC_REG_OFFSET + REG_SEC_TEMP_KEY4)
#define MAC_REG_SEC_TEMP_KEY5			(MAC_REG_OFFSET + REG_SEC_TEMP_KEY5)
#define MAC_REG_SEC_TEMP_KEY6			(MAC_REG_OFFSET + REG_SEC_TEMP_KEY6)
#define MAC_REG_SEC_TEMP_KEY7			(MAC_REG_OFFSET + REG_SEC_TEMP_KEY7)
#define MAC_REG_SEC_RSC0			(MAC_REG_OFFSET + REG_SEC_RSC0)
#define MAC_REG_SEC_RSC1			(MAC_REG_OFFSET + REG_SEC_RSC1)
#define MAC_REG_SEC_RDATA_TAG0			(MAC_REG_OFFSET + REG_SEC_RDATA_TAG0)
#define MAC_REG_SEC_RDATA_TAG1			(MAC_REG_OFFSET + REG_SEC_RDATA_TAG1)
#define MAC_REG_SEC_RDATA_INFO0			(MAC_REG_OFFSET + REG_SEC_RDATA_INFO0)
#define MAC_REG_SEC_RDATA_INFO1			(MAC_REG_OFFSET + REG_SEC_RDATA_INFO1)
#define MAC_REG_SEC_RDATA_INFO2			(MAC_REG_OFFSET + REG_SEC_RDATA_INFO2)
#define MAC_REG_SEC_RDATA_INFO3			(MAC_REG_OFFSET + REG_SEC_RDATA_INFO3)
#define MAC_REG_SEC_RDATA_INFO4			(MAC_REG_OFFSET + REG_SEC_RDATA_INFO4)
#define MAC_REG_SEC_RDATA_INFO5			(MAC_REG_OFFSET + REG_SEC_RDATA_INFO5)
#define MAC_REG_SEC_RDATA_INFO6			(MAC_REG_OFFSET + REG_SEC_RDATA_INFO6)
#define MAC_REG_SEC_RDATA_INFO7			(MAC_REG_OFFSET + REG_SEC_RDATA_INFO7)
#define MAC_REG_SEC_RDATA_INFO8			(MAC_REG_OFFSET + REG_SEC_RDATA_INFO8)
#define MAC_REG_SEC_RDATA_INFO9			(MAC_REG_OFFSET + REG_SEC_RDATA_INFO9)
#define MAC_REG_SEC_RDATA_INFO10		(MAC_REG_OFFSET + REG_SEC_RDATA_INFO10)
#define MAC_REG_HE_TB_PWR01			(MAC_REG_OFFSET + REG_HE_TB_PWR01)
#define MAC_REG_HE_TB_PWR23			(MAC_REG_OFFSET + REG_HE_TB_PWR23)
#define MAC_REG_HE_TB_PWR45			(MAC_REG_OFFSET + REG_HE_TB_PWR45)
#define MAC_REG_HE_TB_PWR67			(MAC_REG_OFFSET + REG_HE_TB_PWR67)
#define MAC_REG_HE_TB_PWR89			(MAC_REG_OFFSET + REG_HE_TB_PWR89)
#define MAC_REG_HE_TB_PWR1011			(MAC_REG_OFFSET + REG_HE_TB_PWR1011)
#define MAC_REG_OBSS_PD_TX_PWR_OPT		(MAC_REG_OFFSET + REG_OBSS_PD_TX_PWR_OPT)
#define MAC_REG_PHYRX_DELAY			(MAC_REG_OFFSET + REG_PHYRX_DELAY)
#define MAC_REG_AID_CMD				(MAC_REG_OFFSET + REG_AID_CMD)
#define MAC_REG_AID_STATUS			(MAC_REG_OFFSET + REG_AID_STATUS)
#define MAC_REG_AID_MAC_ADDR0			(MAC_REG_OFFSET + REG_AID_MAC_ADDR0)
#define MAC_REG_AID_MAC_ADDR1			(MAC_REG_OFFSET + REG_AID_MAC_ADDR1)
#define MAC_REG_AID_MAC_AID			(MAC_REG_OFFSET + REG_AID_MAC_AID)
#define MAC_REG_AID_INFO0			(MAC_REG_OFFSET + REG_AID_INFO0)
#define MAC_REG_AID_INFO1			(MAC_REG_OFFSET + REG_AID_INFO1)
#define MAC_REG_OBSS_SR_NON_SRG_CFG		(MAC_REG_OFFSET + REG_OBSS_SR_NON_SRG_CFG)
#define MAC_REG_OBSS_SR_NON_SRG_COLOR_EN0	(MAC_REG_OFFSET + REG_OBSS_SR_NON_SRG_COLOR_EN0)
#define MAC_REG_OBSS_SR_NON_SRG_COLOR_EN1	(MAC_REG_OFFSET + REG_OBSS_SR_NON_SRG_COLOR_EN1)
#define MAC_REG_OBSS_SR_NON_SRG_BSSID_EN0	(MAC_REG_OFFSET + REG_OBSS_SR_NON_SRG_BSSID_EN0)
#define MAC_REG_OBSS_SR_NON_SRG_BSSID_EN1	(MAC_REG_OFFSET + REG_OBSS_SR_NON_SRG_BSSID_EN1)
#define MAC_REG_SRG_CFG				(MAC_REG_OFFSET + REG_SRG_CFG)
#define MAC_REG_SRG_MAP_COLOR0			(MAC_REG_OFFSET + REG_SRG_MAP_COLOR0)
#define MAC_REG_SRG_MAP_COLOR1			(MAC_REG_OFFSET + REG_SRG_MAP_COLOR1)
#define MAC_REG_SRG_MAP_BSSID0			(MAC_REG_OFFSET + REG_SRG_MAP_BSSID0)
#define MAC_REG_SRG_MAP_BSSID1			(MAC_REG_OFFSET + REG_SRG_MAP_BSSID1)
#define MAC_REG_SRG_SR_COLOR_EN0		(MAC_REG_OFFSET + REG_SRG_SR_COLOR_EN0)
#define MAC_REG_SRG_SR_COLOR_EN1		(MAC_REG_OFFSET + REG_SRG_SR_COLOR_EN1)
#define MAC_REG_SRG_SR_BSSID_EN0		(MAC_REG_OFFSET + REG_SRG_SR_BSSID_EN0)
#define MAC_REG_SRG_SR_BSSID_EN1		(MAC_REG_OFFSET + REG_SRG_SR_BSSID_EN1)
#define MAC_REG_SR_CFG				(MAC_REG_OFFSET + REG_SR_CFG)
#define MAC_REG_CTS_TX_PWR			(MAC_REG_OFFSET + REG_CTS_TX_PWR)
#define MAC_REG_DBG_TX_SIFS_RESP_CNT		(MAC_REG_OFFSET + REG_DBG_TX_SIFS_RESP_CNT)
#define MAC_REG_DBG_SIG				(MAC_REG_OFFSET + REG_DBG_SIG)
#define MAC_REG_DBG_MPDU_FLOW			(MAC_REG_OFFSET + REG_DBG_MPDU_FLOW)
#define MAC_REG_DBG_PPDU_TYPE0			(MAC_REG_OFFSET + REG_DBG_PPDU_TYPE0)
#define MAC_REG_DBG_PPDU_TYPE1			(MAC_REG_OFFSET + REG_DBG_PPDU_TYPE1)
#define MAC_REG_DBG_PPDU_TYPE2			(MAC_REG_OFFSET + REG_DBG_PPDU_TYPE2)
#define MAC_REG_OPT_RSVD			(MAC_REG_OFFSET + REG_OPT_RSVD)
#define MAC_REG_OPT_RSVD1			(MAC_REG_OFFSET + REG_OPT_RSVD1)
#define MAC_REG_DBG_SIM_CFG_OPT			(MAC_REG_OFFSET + REG_DBG_SIM_CFG_OPT)
#define MAC_REG_DBG_RXE_STATE			(MAC_REG_OFFSET + REG_DBG_RXE_STATE)
#define MAC_REG_DBG_RXE_TID_UPD_STATE		(MAC_REG_OFFSET + REG_DBG_RXE_TID_UPD_STATE)
#define MAC_REG_DBG_TXE_STATE			(MAC_REG_OFFSET + REG_DBG_TXE_STATE)
#define MAC_REG_DBG_DMA_STATE			(MAC_REG_OFFSET + REG_DBG_DMA_STATE)
#define MAC_REG_DBG_SEC_STATE			(MAC_REG_OFFSET + REG_DBG_SEC_STATE)
#define MAC_REG_DBG_QUE_READ01			(MAC_REG_OFFSET + REG_DBG_QUE_READ01)
#define MAC_REG_DBG_QUE_READ23			(MAC_REG_OFFSET + REG_DBG_QUE_READ23)
#define MAC_REG_DBG_QUE_READ45			(MAC_REG_OFFSET + REG_DBG_QUE_READ45)
#define MAC_REG_DBG_QUE_READ67			(MAC_REG_OFFSET + REG_DBG_QUE_READ67)
#define MAC_REG_DBG_QUE_READ89			(MAC_REG_OFFSET + REG_DBG_QUE_READ89)
#define MAC_REG_DBG_TX_PPDU_CNT01		(MAC_REG_OFFSET + REG_DBG_TX_PPDU_CNT01)
#define MAC_REG_DBG_TX_PPDU_CNT23		(MAC_REG_OFFSET + REG_DBG_TX_PPDU_CNT23)
#define MAC_REG_DBG_TX_PPDU_CNT45		(MAC_REG_OFFSET + REG_DBG_TX_PPDU_CNT45)
#define MAC_REG_DBG_TX_PPDU_CNT67		(MAC_REG_OFFSET + REG_DBG_TX_PPDU_CNT67)
#define MAC_REG_DBG_TX_PPDU_CNT89		(MAC_REG_OFFSET + REG_DBG_TX_PPDU_CNT89)
#define MAC_REG_DBG_TX_MPDU_SEC01		(MAC_REG_OFFSET + REG_DBG_TX_MPDU_SEC01)
#define MAC_REG_DBG_TX_MPDU_SEC23		(MAC_REG_OFFSET + REG_DBG_TX_MPDU_SEC23)
#define MAC_REG_DBG_TX_MPDU_SEC45		(MAC_REG_OFFSET + REG_DBG_TX_MPDU_SEC45)
#define MAC_REG_DBG_TX_MPDU_SEC67		(MAC_REG_OFFSET + REG_DBG_TX_MPDU_SEC67)
#define MAC_REG_DBG_TX_MPDU_SEC89		(MAC_REG_OFFSET + REG_DBG_TX_MPDU_SEC89)
#define MAC_REG_DBG_TX_MPDU_NOSEC01		(MAC_REG_OFFSET + REG_DBG_TX_MPDU_NOSEC01)
#define MAC_REG_DBG_TX_MPDU_NOSEC23		(MAC_REG_OFFSET + REG_DBG_TX_MPDU_NOSEC23)
#define MAC_REG_DBG_TX_MPDU_NOSEC45		(MAC_REG_OFFSET + REG_DBG_TX_MPDU_NOSEC45)
#define MAC_REG_DBG_TX_MPDU_NOSEC67		(MAC_REG_OFFSET + REG_DBG_TX_MPDU_NOSEC67)
#define MAC_REG_DBG_TX_MPDU_NOSEC89		(MAC_REG_OFFSET + REG_DBG_TX_MPDU_NOSEC89)
#define MAC_REG_DBG_ERR_CNT0			(MAC_REG_OFFSET + REG_DBG_ERR_CNT0)
#define MAC_REG_DBG_PPDU_FLOW			(MAC_REG_OFFSET + REG_DBG_PPDU_FLOW)
#define MAC_REG_DBG_RX_PPDU_CNT0		(MAC_REG_OFFSET + REG_DBG_RX_PPDU_CNT0)
#define MAC_REG_DBG_RX_PPDU_CNT1		(MAC_REG_OFFSET + REG_DBG_RX_PPDU_CNT1)
#define MAC_REG_DBG_RX_PPDU_CNT2		(MAC_REG_OFFSET + REG_DBG_RX_PPDU_CNT2)
#define MAC_REG_DBG_RX_PPDU_CNT3		(MAC_REG_OFFSET + REG_DBG_RX_PPDU_CNT3)
#define MAC_REG_DBG_RX_PPDU_CNT4		(MAC_REG_OFFSET + REG_DBG_RX_PPDU_CNT4)
#define MAC_REG_DBG_RX_PPDU_CNT5		(MAC_REG_OFFSET + REG_DBG_RX_PPDU_CNT5)
#define MAC_REG_DBG_RX_PPDU_CNT6		(MAC_REG_OFFSET + REG_DBG_RX_PPDU_CNT6)
#define MAC_REG_DBG_INTR_CNT0			(MAC_REG_OFFSET + REG_DBG_INTR_CNT0)
#define MAC_REG_DBG_INTR_CNT1			(MAC_REG_OFFSET + REG_DBG_INTR_CNT1)
#define MAC_REG_DBG_INTR_CNT2			(MAC_REG_OFFSET + REG_DBG_INTR_CNT2)
#define MAC_REG_DBG_INTR_CNT3			(MAC_REG_OFFSET + REG_DBG_INTR_CNT3)
#define MAC_REG_DBG_INTR_CNT4			(MAC_REG_OFFSET + REG_DBG_INTR_CNT4)
#define MAC_REG_DBG_INTR_CNT5			(MAC_REG_OFFSET + REG_DBG_INTR_CNT5)
#define MAC_REG_DBG_INTR_CNT6			(MAC_REG_OFFSET + REG_DBG_INTR_CNT6)
#define MAC_REG_DBG_LAST_TXVEC0			(MAC_REG_OFFSET + REG_DBG_LAST_TXVEC0)
#define MAC_REG_DBG_LAST_TXVEC1			(MAC_REG_OFFSET + REG_DBG_LAST_TXVEC1)
#define MAC_REG_DBG_LAST_TXVEC2			(MAC_REG_OFFSET + REG_DBG_LAST_TXVEC2)
#define MAC_REG_DBG_LAST_TXVEC3			(MAC_REG_OFFSET + REG_DBG_LAST_TXVEC3)
#define MAC_REG_DBG_LAST_TXVEC4			(MAC_REG_OFFSET + REG_DBG_LAST_TXVEC4)
#define MAC_REG_DBG_LAST_TXVEC5			(MAC_REG_OFFSET + REG_DBG_LAST_TXVEC5)
#define MAC_REG_DBG_LAST_TXVEC6			(MAC_REG_OFFSET + REG_DBG_LAST_TXVEC6)
#define MAC_REG_DBG_LAST_RXVEC0			(MAC_REG_OFFSET + REG_DBG_LAST_RXVEC0)
#define MAC_REG_DBG_LAST_RXVEC1			(MAC_REG_OFFSET + REG_DBG_LAST_RXVEC1)
#define MAC_REG_DBG_LAST_RXVEC2			(MAC_REG_OFFSET + REG_DBG_LAST_RXVEC2)
#define MAC_REG_DBG_LAST_RXVEC3			(MAC_REG_OFFSET + REG_DBG_LAST_RXVEC3)
#define MAC_REG_DBG_LAST_RXVEC4			(MAC_REG_OFFSET + REG_DBG_LAST_RXVEC4)
#define MAC_REG_DBG_LAST_RXVEC5			(MAC_REG_OFFSET + REG_DBG_LAST_RXVEC5)
#define MAC_REG_DBG_LAST_RXVEC6			(MAC_REG_OFFSET + REG_DBG_LAST_RXVEC6)
#define MAC_REG_DBG_LAST_RXVEC7			(MAC_REG_OFFSET + REG_DBG_LAST_RXVEC7)
#define MAC_REG_DBG_LAST_RXVEC8			(MAC_REG_OFFSET + REG_DBG_LAST_RXVEC8)
#define MAC_REG_DBG_NAV0			(MAC_REG_OFFSET + REG_DBG_NAV0)
#define MAC_REG_DBG_NAV1			(MAC_REG_OFFSET + REG_DBG_NAV1)
#define MAC_REG_DBG_SR0				(MAC_REG_OFFSET + REG_DBG_SR0)
#define MAC_REG_DBG_SR1				(MAC_REG_OFFSET + REG_DBG_SR1)
#define MAC_REG_DBG_LAT_NAV0			(MAC_REG_OFFSET + REG_DBG_LAT_NAV0)
#define MAC_REG_DBG_LAT_NAV1			(MAC_REG_OFFSET + REG_DBG_LAT_NAV1)
#define MAC_REG_DBG_STATE_CFG			(MAC_REG_OFFSET + REG_DBG_STATE_CFG)
#define MAC_REG_DBG_STATE_CHK			(MAC_REG_OFFSET + REG_DBG_STATE_CHK)
#define MAC_REG_DBG_LAST_TX_LEN			(MAC_REG_OFFSET + REG_DBG_LAST_TX_LEN)
#define MAC_REG_DBG_TXDMA_DELAY			(MAC_REG_OFFSET + REG_DBG_TXDMA_DELAY)
#define MAC_REG_DBG_LAST_CBF			(MAC_REG_OFFSET + REG_DBG_LAST_CBF)

/*
 * MAC VIF register macros
 */

#define REGS_VIF_STRIDE	(REG_VIF1_CFG0 - REG_VIF0_CFG0)
#define VIF(n)		((n) * REGS_VIF_STRIDE)

#define MAC_REG_VIF_CFG0(v)			(MAC_REG_VIF0_CFG0 + VIF(v))
#define MAC_REG_VIF_CFG1(v)			(MAC_REG_VIF0_CFG1 + VIF(v))
#define MAC_REG_VIF_RX_FILTER0(v)		(MAC_REG_VIF0_RX_FILTER0 + VIF(v))
#define MAC_REG_VIF_RX_FILTER1(v)		(MAC_REG_VIF0_RX_FILTER1 + VIF(v))
#define MAC_REG_VIF_MAC0_L(v)			(MAC_REG_VIF0_MAC0_L + VIF(v))
#define MAC_REG_VIF_MAC0_H(v)			(MAC_REG_VIF0_MAC0_H + VIF(v))
#define MAC_REG_VIF_MAC1_L(v)			(MAC_REG_VIF0_MAC1_L + VIF(v))
#define MAC_REG_VIF_MAC1_H(v)			(MAC_REG_VIF0_MAC1_H + VIF(v))
#define MAC_REG_VIF_BSS0_L(v)			(MAC_REG_VIF0_BSS0_L + VIF(v))
#define MAC_REG_VIF_BSS0_H(v)			(MAC_REG_VIF0_BSS0_H + VIF(v))
#define MAC_REG_VIF_BSS1_L(v)			(MAC_REG_VIF0_BSS1_L + VIF(v))
#define MAC_REG_VIF_BSS1_H(v)			(MAC_REG_VIF0_BSS1_H + VIF(v))
#define MAC_REG_VIF_AID(v)			(MAC_REG_VIF0_AID + VIF(v))
#define MAC_REG_VIF_BFM0(v)			(MAC_REG_VIF0_BFM0 + VIF(v))
#define MAC_REG_VIF_BFM1(v)			(MAC_REG_VIF0_BFM1 + VIF(v))
#define MAC_REG_VIF_BFM2(v)			(MAC_REG_VIF0_BFM2 + VIF(v))
#define MAC_REG_VIF_TSF_L(v)			(MAC_REG_VIF0_TSF_L + VIF(v))
#define MAC_REG_VIF_TSF_H(v)			(MAC_REG_VIF0_TSF_H + VIF(v))
#define MAC_REG_VIF_TSF_OFFSET_L(v)		(MAC_REG_VIF0_TSF_OFFSET_L + VIF(v))
#define MAC_REG_VIF_TSF_OFFSET_H(v)		(MAC_REG_VIF0_TSF_OFFSET_H + VIF(v))
#define MAC_REG_VIF_TSF_TIME0(v)		(MAC_REG_VIF0_TSF_TIME0 + VIF(v))
#define MAC_REG_VIF_TSF_TIME1(v)		(MAC_REG_VIF0_TSF_TIME1 + VIF(v))
#define MAC_REG_VIF_TRIG_RESP_QUEUE_EN(v)	(MAC_REG_VIF0_TRIG_RESP_QUEUE_EN + VIF(v))
#define MAC_REG_VIF_BSR(v)			(MAC_REG_VIF0_BSR + VIF(v))
#define MAC_REG_VIF_MIN_MPDU_SPACING(v)		(MAC_REG_VIF0_MIN_MPDU_SPACING + VIF(v))
#define MAC_REG_VIF_QNULL_CTRL_AC0(v)		(MAC_REG_VIF0_QNULL_CTRL_AC0 + VIF(v))
#define MAC_REG_VIF_QNULL_CTRL_AC1(v)		(MAC_REG_VIF0_QNULL_CTRL_AC1 + VIF(v))
#define MAC_REG_VIF_QNULL_CTRL_AC2(v)		(MAC_REG_VIF0_QNULL_CTRL_AC2 + VIF(v))
#define MAC_REG_VIF_QNULL_CTRL_AC3(v)		(MAC_REG_VIF0_QNULL_CTRL_AC3 + VIF(v))
#define MAC_REG_VIF_UORA_CFG(v)			(MAC_REG_VIF0_UORA_CFG + VIF(v))
#define MAC_REG_VIF_UORA_CFG_STATUS(v)		(MAC_REG_VIF0_UORA_CFG_STATUS + VIF(v))
#define MAC_REG_VIF_UORA_CFG1(v)		(MAC_REG_VIF0_UORA_CFG1 + VIF(v))
#define MAC_REG_VIF_NON_SRG_OBSS_PD(v)		(MAC_REG_VIF0_NON_SRG_OBSS_PD + VIF(v))
#define MAC_REG_VIF_SRG_OBSS_PD(v)		(MAC_REG_VIF0_SRG_OBSS_PD + VIF(v))
#define MAC_REG_VIF_TX_PWR(v)			(MAC_REG_VIF0_TX_PWR + VIF(v))
#define MAC_REG_VIF_CFG_INFO(v)			(MAC_REG_VIF0_CFG_INFO + VIF(v))
#define MAC_REG_VIF_MBSSID_MASK_L32(v)		(MAC_REG_VIF0_MBSSID_MASK_L32 + VIF(v))
#define MAC_REG_VIF_MBSSID_MASK_H16(v)		(MAC_REG_VIF0_MBSSID_MASK_H16 + VIF(v))
#define MAC_REG_VIF_TRIG(v)			(MAC_REG_VIF0_TRIG + VIF(v))
#define MAC_REG_VIF_ESP(v)			(MAC_REG_VIF0_ESP + VIF(v))
#define MAC_REG_VIF_ESP_STATUS(v)		(MAC_REG_VIF0_ESP_STATUS + VIF(v))
#define MAC_REG_VIF_HTC_OMI(v)			(MAC_REG_VIF0_HTC_OMI + VIF(v))
#define MAC_REG_VIF_BASIC_TRIG_USER_INFO(v)	(MAC_REG_VIF0_BASIC_TRIG_USER_INFO + VIF(v))
#define MAC_REG_VIF_BASIC_TRIG_PSDU_LEN(v)	(MAC_REG_VIF0_BASIC_TRIG_PSDU_LEN + VIF(v))
#define MAC_REG_VIF_RSVD(v)			(MAC_REG_VIF0_RSVD + VIF(v))

/*
 * Register field definitions for <linux/bitfield.h>
 * #define VIF_CFG0_EN		VIF0_CFG0_EN_MASK
 */

#define INTR_ENABLE_TX_DONE			INTR_ENABLE_TX_DONE_MASK
#define INTR_ENABLE_MCU				INTR_ENABLE_MCU_MASK
#define INTR_ENABLE_RX_DONE			INTR_ENABLE_RX_DONE_MASK
#define INTR_ENABLE_RESP_ESP			INTR_ENABLE_RESP_ESP_MASK
#define INTR_ENABLE_RX_ESP			INTR_ENABLE_RX_ESP_MASK
#define INTR_ENABLE_NOMORE_TF_ESP		INTR_ENABLE_NOMORE_TF_ESP_MASK
#define INTR_ENABLE_NOMORE_RARU_ESP		INTR_ENABLE_NOMORE_RARU_ESP_MASK
#define INTR_ENABLE_BASIC_TRIG_QNULL_VIF	INTR_ENABLE_BASIC_TRIG_QNULL_VIF_MASK
#define INTR_ENABLE_TSF0			INTR_ENABLE_TSF0_MASK
#define INTR_ENABLE_TSF1			INTR_ENABLE_TSF1_MASK
#define INTR_ENABLE_RX_BUF			INTR_ENABLE_RX_BUF_MASK
#define INTR_ENABLE_RX_DTIM			INTR_ENABLE_RX_DTIM_MASK
#define INTR_ENABLE_COEX			INTR_ENABLE_COEX_MASK
#define INTR_CLEAR_TX_DONE			INTR_CLEAR_TX_DONE_MASK
#define INTR_CLEAR_MCU				INTR_CLEAR_MCU_MASK
#define INTR_CLEAR_RX_DONE			INTR_CLEAR_RX_DONE_MASK
#define INTR_CLEAR_RESP_ESP			INTR_CLEAR_RESP_ESP_MASK
#define INTR_CLEAR_RX_ESP			INTR_CLEAR_RX_ESP_MASK
#define INTR_CLEAR_NOMORE_TF_ESP		INTR_CLEAR_NOMORE_TF_ESP_MASK
#define INTR_CLEAR_NOMORE_RARU_ESP		INTR_CLEAR_NOMORE_RARU_ESP_MASK
#define INTR_CLEAR_BASIC_TRIG_QNULL_VIF		INTR_CLEAR_BASIC_TRIG_QNULL_VIF_MASK
#define INTR_CLEAR_TSF0				INTR_CLEAR_TSF0_MASK
#define INTR_CLEAR_TSF1				INTR_CLEAR_TSF1_MASK
#define INTR_CLEAR_RX_BUF			INTR_CLEAR_RX_BUF_MASK
#define INTR_CLEAR_RX_DTIM			INTR_CLEAR_RX_DTIM_MASK
#define INTR_CLEAR_COEX				INTR_CLEAR_COEX_MASK
#define INTR_STATUS_TX_DONE			INTR_STATUS_TX_DONE_MASK
#define INTR_STATUS_MCU				INTR_STATUS_MCU_MASK
#define INTR_STATUS_RX_DONE			INTR_STATUS_RX_DONE_MASK
#define INTR_STATUS_RESP_ESP			INTR_STATUS_RESP_ESP_MASK
#define INTR_STATUS_RX_ESP			INTR_STATUS_RX_ESP_MASK
#define INTR_STATUS_NOMORE_TF_ESP		INTR_STATUS_NOMORE_TF_ESP_MASK
#define INTR_STATUS_NOMORE_RARU_ESP		INTR_STATUS_NOMORE_RARU_ESP_MASK
#define INTR_STATUS_BASIC_TRIG_QNULL_VIF	INTR_STATUS_BASIC_TRIG_QNULL_VIF_MASK
#define INTR_STATUS_TSF0			INTR_STATUS_TSF0_MASK
#define INTR_STATUS_TSF1			INTR_STATUS_TSF1_MASK
#define INTR_STATUS_RX_BUF			INTR_STATUS_RX_BUF_MASK
#define INTR_STATUS_RX_DTIM			INTR_STATUS_RX_DTIM_MASK
#define INTR_STATUS_COEX			INTR_STATUS_COEX_MASK
#define MAC_CFG_RST				MAC_CFG_RST_MASK
#define MAC_CFG_EN				MAC_CFG_EN_MASK
#define MAC_STATUS_DMA_BUSY			MAC_STATUS_DMA_BUSY_MASK
#define MAC_STATUS_RXE_BUSY			MAC_STATUS_RXE_BUSY_MASK
#define MCU_CFG_RSTB				MCU_CFG_RSTB_MASK
#define MCU_STATUS_DONE				MCU_STATUS_DONE_MASK
#define VER_INFO_MAC_VER_INFO			VER_INFO_MAC_VER_INFO_MASK
#define GIT_VER_MAC_GIT_VER			GIT_VER_MAC_GIT_VER_MASK
#define DEV_CFG_BAND				DEV_CFG_BAND_MASK
#define DEV_CFG_ONAV_MU_OPT			DEV_CFG_ONAV_MU_OPT_MASK
#define DEV_CFG_SLOT_TIME			DEV_CFG_SLOT_TIME_MASK
#define DEV_CFG_N_RX				DEV_CFG_N_RX_MASK
#define DEV_CFG_CBF_EOF				DEV_CFG_CBF_EOF_MASK
#define DEV_CFG_HETB_CBF_RU_CHK			DEV_CFG_HETB_CBF_RU_CHK_MASK
#define DEV_CFG_RX_FILTER_OUT_ABORT_EN		DEV_CFG_RX_FILTER_OUT_ABORT_EN_MASK
#define DEV_CFG_RX_EOF_DEL_ABORT_EN		DEV_CFG_RX_EOF_DEL_ABORT_EN_MASK
#define DEV_CFG_NON_BASIC_TRIG_CONT_OPT		DEV_CFG_NON_BASIC_TRIG_CONT_OPT_MASK
#define DEV_CFG_ACK_ENABLED_AMDPU_IN_HETB	DEV_CFG_ACK_ENABLED_AMDPU_IN_HETB_MASK
#define DEV_CFG_ULMU_DATA_DIS_QOSNULL_EN	DEV_CFG_ULMU_DATA_DIS_QOSNULL_EN_MASK
#define DEV_CFG_BSRP_UL_MU_DATA_CHK_EN		DEV_CFG_BSRP_UL_MU_DATA_CHK_EN_MASK
#define DEV_CFG_UORA_CONT_DATA_CHK_EN		DEV_CFG_UORA_CONT_DATA_CHK_EN_MASK
#define DEV_CFG_WIDE_RU_SUPPORT_20		DEV_CFG_WIDE_RU_SUPPORT_20_MASK
#define DEV_CFG_WIDE_RU_SUPPORT			DEV_CFG_WIDE_RU_SUPPORT_MASK
#define DEV_CFG_EN_LAST_FRAG_BITMAP_UPD		DEV_CFG_EN_LAST_FRAG_BITMAP_UPD_MASK
#define DEV_CFG_HETB_TID_CHK_IG_NOACK		DEV_CFG_HETB_TID_CHK_IG_NOACK_MASK
#define DEV_CFG_UORA_STA40_PRI_UL80_160		DEV_CFG_UORA_STA40_PRI_UL80_160_MASK
#define DEV_CFG_UORA_STA40_SEC80		DEV_CFG_UORA_STA40_SEC80_MASK
#define DEV_CFG_CBF_RESP_NDP_CHK_EN		DEV_CFG_CBF_RESP_NDP_CHK_EN_MASK
#define DEV_OPT1_BQRP_EN			DEV_OPT1_BQRP_EN_MASK
#define DEV_OPT1_NFRP_EN			DEV_OPT1_NFRP_EN_MASK
#define DEV_OPT1_PHYRX_TD_DELAY			DEV_OPT1_PHYRX_TD_DELAY_MASK
#define DEV_OPT1_NONHT_1OFDM_US_DELAY		DEV_OPT1_NONHT_1OFDM_US_DELAY_MASK
#define DEV_OPT1_NONHT_1OFDM_CYCLE_DELAY	DEV_OPT1_NONHT_1OFDM_CYCLE_DELAY_MASK
#define DEV_CS_IGNORE_CCA			DEV_CS_IGNORE_CCA_MASK
#define DEV_CS_IGNORE_RX			DEV_CS_IGNORE_RX_MASK
#define DEV_CS_BUSY_FLT_RX			DEV_CS_BUSY_FLT_RX_MASK
#define DEV_CS_IGNORE_RX_SYNC			DEV_CS_IGNORE_RX_SYNC_MASK
#define DEV_CS_IGNORE_RX_TX_WAIT_RESP		DEV_CS_IGNORE_RX_TX_WAIT_RESP_MASK
#define DEV_CS_BUSY_COEX			DEV_CS_BUSY_COEX_MASK
#define DEV_CH_COEX_WLAN_CHANNEL_FREQ		DEV_CH_COEX_WLAN_CHANNEL_FREQ_MASK
#define DEV_CH_COEX_WLAN_CHANNEL_OFFSET		DEV_CH_COEX_WLAN_CHANNEL_OFFSET_MASK
#define DEV_CH_COEX_RESP_PRI0			DEV_CH_COEX_RESP_PRI0_MASK
#define DEV_CH_COEX_RESP_PRI1			DEV_CH_COEX_RESP_PRI1_MASK
#define DEV_PMU_SMPS_EN				DEV_PMU_SMPS_EN_MASK
#define DEV_PMU_SMPS_MODE			DEV_PMU_SMPS_MODE_MASK
#define DEV_FORCE_CLR_CLR_FROCE_VEC		DEV_FORCE_CLR_CLR_FROCE_VEC_MASK
#define DEV_FORCE_CLR_CLR_FORCE_EN		DEV_FORCE_CLR_CLR_FORCE_EN_MASK
#define DEV_OPT2_CONT_TX_TERMI_FSMIDLE_EN	DEV_OPT2_CONT_TX_TERMI_FSMIDLE_EN_MASK
#define DEV_OPT2_TXPHY_TO_EN			DEV_OPT2_TXPHY_TO_EN_MASK
#define DEV_OPT2_RTO_TO_CHECK_EN		DEV_OPT2_RTO_TO_CHECK_EN_MASK
#define DEV_OPT2_EDCA_TXVEC_PROT_EN		DEV_OPT2_EDCA_TXVEC_PROT_EN_MASK
#define DEV_OPT2_TXBUSY_PHY_EN			DEV_OPT2_TXBUSY_PHY_EN_MASK
#define DEV_OPT2_RXBUSY_CHK_EN			DEV_OPT2_RXBUSY_CHK_EN_MASK
#define DEV_OPT2_RXBUSY_CONF_CHK_EN		DEV_OPT2_RXBUSY_CONF_CHK_EN_MASK
#define DEV_OPT2_SIFS_LOST_STOP_TX_EN		DEV_OPT2_SIFS_LOST_STOP_TX_EN_MASK
#define DEV_OPT2_QUE_CANCEL_ABORT_TX		DEV_OPT2_QUE_CANCEL_ABORT_TX_MASK
#define DEV_OPT2_HETB_INTR_ACKBA_EN		DEV_OPT2_HETB_INTR_ACKBA_EN_MASK
#define DEV_OPT2_SIFS_LOST_SEND_TX_EN		DEV_OPT2_SIFS_LOST_SEND_TX_EN_MASK
#define DEV_OPT2_AID_MATCH_BASIC_TRIG_END_EN	DEV_OPT2_AID_MATCH_BASIC_TRIG_END_EN_MASK
#define DEV_OPT2_AID_MATCH_BSRP_TRIG_END_EN	DEV_OPT2_AID_MATCH_BSRP_TRIG_END_EN_MASK
#define DEV_OPT2_BRP_CBF_HTC_DIS		DEV_OPT2_BRP_CBF_HTC_DIS_MASK
#define DEV_OPT2_BRP_CBF_HTC_STATIC		DEV_OPT2_BRP_CBF_HTC_STATIC_MASK
#define DEV_OPT2_CBF_SEQ_EN			DEV_OPT2_CBF_SEQ_EN_MASK
#define DEV_OPT2_CBF_ANGLE_ZERO			DEV_OPT2_CBF_ANGLE_ZERO_MASK
#define DEV_OPT2_SND_SEQ_RST_EN			DEV_OPT2_SND_SEQ_RST_EN_MASK
#define DEV_OPT2_COEX_WLAN_ABORT_EN		DEV_OPT2_COEX_WLAN_ABORT_EN_MASK
#define DEV_OPT2_ACKP0_CHECK_EN			DEV_OPT2_ACKP0_CHECK_EN_MASK
#define DEV_OPT2_CHECK_VHT_GROUPID0		DEV_OPT2_CHECK_VHT_GROUPID0_MASK
#define DEV_OPT2_CHECK_VHT_GROUPID63		DEV_OPT2_CHECK_VHT_GROUPID63_MASK
#define DEV_OPT2_TX_CANCEL_QUE_CHK_EN		DEV_OPT2_TX_CANCEL_QUE_CHK_EN_MASK
#define DEV_OPT2_BKEND_QUE_CHK_EN		DEV_OPT2_BKEND_QUE_CHK_EN_MASK
#define DEV_FORCE_QUE_CLR_CLR_FROCE_VEC		DEV_FORCE_QUE_CLR_CLR_FROCE_VEC_MASK
#define DEV_CLK_CFG_HCLK_GATE_EN		DEV_CLK_CFG_HCLK_GATE_EN_MASK
#define DEV_CLK_CFG_HCLKX0_GATE_EN		DEV_CLK_CFG_HCLKX0_GATE_EN_MASK
#define DEV_CLK_CFG_HCLKX1_GATE_EN		DEV_CLK_CFG_HCLKX1_GATE_EN_MASK
#define DEV_CLK_CFG_HCLKX2_GATE_EN		DEV_CLK_CFG_HCLKX2_GATE_EN_MASK
#define INTR_SELECT_INTR_SEL			INTR_SELECT_INTR_SEL_MASK
#define TX_TXTIME_CMD_CMD			TX_TXTIME_CMD_CMD_MASK
#define TX_TXTIME_RESULT_TXTIME			TX_TXTIME_RESULT_TXTIME_MASK
#define TX_TXTIME_RESULT_ERR			TX_TXTIME_RESULT_ERR_MASK
#define TX_TXTIME_TV0_TXVECTOR0			TX_TXTIME_TV0_TXVECTOR0_MASK
#define TX_TXTIME_TV1_TXVECTOR1			TX_TXTIME_TV1_TXVECTOR1_MASK
#define TX_TXTIME_TV2_TXVECTOR2			TX_TXTIME_TV2_TXVECTOR2_MASK
#define TX_TXTIME_TV3_TXVECTOR3			TX_TXTIME_TV3_TXVECTOR3_MASK
#define TX_TXTIME_TV4_TXVECTOR4			TX_TXTIME_TV4_TXVECTOR4_MASK
#define TX_TXTIME_TV5_TXVECTOR5			TX_TXTIME_TV5_TXVECTOR5_MASK
#define TX_TXTIME_TV6_TXVECTOR6			TX_TXTIME_TV6_TXVECTOR6_MASK
#define TX_TO_CFG_TO_EN				TX_TO_CFG_TO_EN_MASK
#define TX_TO_CFG_TO_VALUE			TX_TO_CFG_TO_VALUE_MASK
#define TX_TO_11B_CFG_TO_11B_VALUE		TX_TO_11B_CFG_TO_11B_VALUE_MASK
#define TX_CFG_EARLY_PREP			TX_CFG_EARLY_PREP_MASK
#define TX_HETB_QNULL_RX_BASIC_TRIGGER_FIRST_AC_VIF0 \
	TX_HETB_QNULL_RX_BASIC_TRIGGER_FIRST_AC_VIF0_MASK
#define TX_HETB_QNULL_RX_BASIC_TRIGGER_MU_SPACING_FACTOR_VIF0 \
	TX_HETB_QNULL_RX_BASIC_TRIGGER_MU_SPACING_FACTOR_VIF0_MASK
#define TX_HETB_QNULL_RX_BASIC_TRIGGER_QUE_SEL_VIF0 \
	TX_HETB_QNULL_RX_BASIC_TRIGGER_QUE_SEL_VIF0_MASK
#define TX_HETB_QNULL_RX_BASIC_TRIGGER_TID_LIMIT_VIF0 \
	TX_HETB_QNULL_RX_BASIC_TRIGGER_TID_LIMIT_VIF0_MASK
#define TX_HETB_QNULL_RX_BASIC_TRIGGER_FIRST_AC_VIF1 \
	TX_HETB_QNULL_RX_BASIC_TRIGGER_FIRST_AC_VIF1_MASK
#define TX_HETB_QNULL_RX_BASIC_TRIGGER_MU_SPACING_FACTOR_VIF1 \
	TX_HETB_QNULL_RX_BASIC_TRIGGER_MU_SPACING_FACTOR_VIF1_MASK
#define TX_HETB_QNULL_RX_BASIC_TRIGGER_QUE_SEL_VIF1 \
	TX_HETB_QNULL_RX_BASIC_TRIGGER_QUE_SEL_VIF1_MASK
#define TX_HETB_QNULL_RX_BASIC_TRIGGER_TID_LIMIT_VIF1 \
	TX_HETB_QNULL_RX_BASIC_TRIGGER_TID_LIMIT_VIF1_MASK
#define RX_CFG_PRO				RX_CFG_PRO_MASK
#define RX_BUF_CFG_EN				RX_BUF_CFG_EN_MASK
#define RX_BUF_CFG_BUF_NUM			RX_BUF_CFG_BUF_NUM_MASK
#define RX_BUF_CFG_DMA_EN			RX_BUF_CFG_DMA_EN_MASK
#define RX_BUF_PTR0_READY_PTR			RX_BUF_PTR0_READY_PTR_MASK
#define RX_BUF_PTR0_READ_PTR			RX_BUF_PTR0_READ_PTR_MASK
#define RX_BUF_PTR1_WRITE_PTR			RX_BUF_PTR1_WRITE_PTR_MASK
#define RX_SCB_CTRL_SCB_RESET			RX_SCB_CTRL_SCB_RESET_MASK
#define RX_SCB_CTRL_SCB_WINDOW			RX_SCB_CTRL_SCB_WINDOW_MASK
#define RX_SCB_CTRL_SCB_IGNORE_OVERRUN		RX_SCB_CTRL_SCB_IGNORE_OVERRUN_MASK
#define RX_TO_CFG_RX_TO_VALUE			RX_TO_CFG_RX_TO_VALUE_MASK
#define RX_TO_CFG_RX_TO_PPDU_END_EN		RX_TO_CFG_RX_TO_PPDU_END_EN_MASK
#define RX_TO_11B_CFG_RX_TO_11B_VALUE		RX_TO_11B_CFG_RX_TO_11B_VALUE_MASK
#define RX_DTIM_CFG_DTIM_MODE			RX_DTIM_CFG_DTIM_MODE_MASK
#define RX_DTIM_CFG_DTIM_FCS_CHECK		RX_DTIM_CFG_DTIM_FCS_CHECK_MASK
#define RX_DTIM_CFG_DTIM_SOFT_RST		RX_DTIM_CFG_DTIM_SOFT_RST_MASK
#define RX_DTIM_INFO_RX_FCS			RX_DTIM_INFO_RX_FCS_MASK
#define RX_DTIM_INFO_RX_DTIM_BITMAP_HIT		RX_DTIM_INFO_RX_DTIM_BITMAP_HIT_MASK
#define RX_DTIM_INFO_RX_DTIM_BITMAP_NULL	RX_DTIM_INFO_RX_DTIM_BITMAP_NULL_MASK
#define RX_DTIM_INFO_RX_DTIM_BITMAP_NULL_UNDER	RX_DTIM_INFO_RX_DTIM_BITMAP_NULL_UNDER_MASK
#define RX_DTIM_INFO_RX_DTIM_BITMAP_NULL_ABOVE	RX_DTIM_INFO_RX_DTIM_BITMAP_NULL_ABOVE_MASK
#define RX_DTIM_INFO_RX_IS_DTIM			RX_DTIM_INFO_RX_IS_DTIM_MASK
#define RX_DTIM_INFO_RX_TRAFFIC_INDICATOR	RX_DTIM_INFO_RX_TRAFFIC_INDICATOR_MASK
#define RX_DTIM_INFO_RX_DTIM_AID0_BITMAP_HIT	RX_DTIM_INFO_RX_DTIM_AID0_BITMAP_HIT_MASK
#define RX_DTIM_INFO_RX_BEACON_INTERVAL		RX_DTIM_INFO_RX_BEACON_INTERVAL_MASK
#define SEC_KEY_CMD_KCMD			SEC_KEY_CMD_KCMD_MASK
#define SEC_KEY_CMD_KID				SEC_KEY_CMD_KID_MASK
#define SEC_KEY_CMD_KTYPE			SEC_KEY_CMD_KTYPE_MASK
#define SEC_KEY_CMD_CTYPE			SEC_KEY_CMD_CTYPE_MASK
#define SEC_KEY_CMD_SPP				SEC_KEY_CMD_SPP_MASK
#define SEC_KEY_CMD_IDX				SEC_KEY_CMD_IDX_MASK
#define SEC_KEY_STATUS_KEY_VALID		SEC_KEY_STATUS_KEY_VALID_MASK
#define SEC_MAC_ADDR_L32_MAC_ADDR_32_0		SEC_MAC_ADDR_L32_MAC_ADDR_32_0_MASK
#define SEC_MAC_ADDR_H16_MAC_ADDR_47_32		SEC_MAC_ADDR_H16_MAC_ADDR_47_32_MASK
#define SEC_TEMP_KEY0_TEMP_KEY_31_0		SEC_TEMP_KEY0_TEMP_KEY_31_0_MASK
#define SEC_TEMP_KEY1_TEMP_KEY_63_32		SEC_TEMP_KEY1_TEMP_KEY_63_32_MASK
#define SEC_TEMP_KEY2_TEMP_KEY_95_64		SEC_TEMP_KEY2_TEMP_KEY_95_64_MASK
#define SEC_TEMP_KEY3_TEMP_KEY_127_96		SEC_TEMP_KEY3_TEMP_KEY_127_96_MASK
#define SEC_TEMP_KEY4_TEMP_KEY_159_128		SEC_TEMP_KEY4_TEMP_KEY_159_128_MASK
#define SEC_TEMP_KEY5_TEMP_KEY_191_160		SEC_TEMP_KEY5_TEMP_KEY_191_160_MASK
#define SEC_TEMP_KEY6_TEMP_KEY_223_192		SEC_TEMP_KEY6_TEMP_KEY_223_192_MASK
#define SEC_TEMP_KEY7_TEMP_KEY_255_224		SEC_TEMP_KEY7_TEMP_KEY_255_224_MASK
#define SEC_RSC0_RSC_31_0			SEC_RSC0_RSC_31_0_MASK
#define SEC_RSC1_RSC_63_32			SEC_RSC1_RSC_63_32_MASK
#define SEC_RDATA_TAG0_MAC_ADDR_31_0		SEC_RDATA_TAG0_MAC_ADDR_31_0_MASK
#define SEC_RDATA_TAG1_MAC_ADDR_47_32		SEC_RDATA_TAG1_MAC_ADDR_47_32_MASK
#define SEC_RDATA_TAG1_KID			SEC_RDATA_TAG1_KID_MASK
#define SEC_RDATA_TAG1_KTYPE			SEC_RDATA_TAG1_KTYPE_MASK
#define SEC_RDATA_INFO0_TEMP_KEY_31_0		SEC_RDATA_INFO0_TEMP_KEY_31_0_MASK
#define SEC_RDATA_INFO1_TEMP_KEY_63_32		SEC_RDATA_INFO1_TEMP_KEY_63_32_MASK
#define SEC_RDATA_INFO2_TEMP_KEY_95_64		SEC_RDATA_INFO2_TEMP_KEY_95_64_MASK
#define SEC_RDATA_INFO3_TEMP_KEY_127_96		SEC_RDATA_INFO3_TEMP_KEY_127_96_MASK
#define SEC_RDATA_INFO4_TEMP_KEY_159_128	SEC_RDATA_INFO4_TEMP_KEY_159_128_MASK
#define SEC_RDATA_INFO5_TEMP_KEY_191_160	SEC_RDATA_INFO5_TEMP_KEY_191_160_MASK
#define SEC_RDATA_INFO6_TEMP_KEY_223_192	SEC_RDATA_INFO6_TEMP_KEY_223_192_MASK
#define SEC_RDATA_INFO7_TEMP_KEY_255_224	SEC_RDATA_INFO7_TEMP_KEY_255_224_MASK
#define SEC_RDATA_INFO8_RSC_31_0		SEC_RDATA_INFO8_RSC_31_0_MASK
#define SEC_RDATA_INFO9_RSC_63_32		SEC_RDATA_INFO9_RSC_63_32_MASK
#define SEC_RDATA_INFO10_CTYPE			SEC_RDATA_INFO10_CTYPE_MASK
#define SEC_RDATA_INFO10_SPP			SEC_RDATA_INFO10_SPP_MASK
#define HE_TB_PWR01_MCS0_MIN_PWR		HE_TB_PWR01_MCS0_MIN_PWR_MASK
#define HE_TB_PWR01_MCS0_MAX_PWR		HE_TB_PWR01_MCS0_MAX_PWR_MASK
#define HE_TB_PWR01_MCS1_MIN_PWR		HE_TB_PWR01_MCS1_MIN_PWR_MASK
#define HE_TB_PWR01_MCS1_MAX_PWR		HE_TB_PWR01_MCS1_MAX_PWR_MASK
#define HE_TB_PWR23_MCS2_MIN_PWR		HE_TB_PWR23_MCS2_MIN_PWR_MASK
#define HE_TB_PWR23_MCS2_MAX_PWR		HE_TB_PWR23_MCS2_MAX_PWR_MASK
#define HE_TB_PWR23_MCS3_MIN_PWR		HE_TB_PWR23_MCS3_MIN_PWR_MASK
#define HE_TB_PWR23_MCS3_MAX_PWR		HE_TB_PWR23_MCS3_MAX_PWR_MASK
#define HE_TB_PWR45_MCS4_MIN_PWR		HE_TB_PWR45_MCS4_MIN_PWR_MASK
#define HE_TB_PWR45_MCS4_MAX_PWR		HE_TB_PWR45_MCS4_MAX_PWR_MASK
#define HE_TB_PWR45_MCS5_MIN_PWR		HE_TB_PWR45_MCS5_MIN_PWR_MASK
#define HE_TB_PWR45_MCS5_MAX_PWR		HE_TB_PWR45_MCS5_MAX_PWR_MASK
#define HE_TB_PWR67_MCS6_MIN_PWR		HE_TB_PWR67_MCS6_MIN_PWR_MASK
#define HE_TB_PWR67_MCS6_MAX_PWR		HE_TB_PWR67_MCS6_MAX_PWR_MASK
#define HE_TB_PWR67_MCS7_MIN_PWR		HE_TB_PWR67_MCS7_MIN_PWR_MASK
#define HE_TB_PWR67_MCS7_MAX_PWR		HE_TB_PWR67_MCS7_MAX_PWR_MASK
#define HE_TB_PWR89_MCS8_MIN_PWR		HE_TB_PWR89_MCS8_MIN_PWR_MASK
#define HE_TB_PWR89_MCS8_MAX_PWR		HE_TB_PWR89_MCS8_MAX_PWR_MASK
#define HE_TB_PWR89_MCS9_MIN_PWR		HE_TB_PWR89_MCS9_MIN_PWR_MASK
#define HE_TB_PWR89_MCS9_MAX_PWR		HE_TB_PWR89_MCS9_MAX_PWR_MASK
#define HE_TB_PWR1011_MCS10_MIN_PWR		HE_TB_PWR1011_MCS10_MIN_PWR_MASK
#define HE_TB_PWR1011_MCS10_MAX_PWR		HE_TB_PWR1011_MCS10_MAX_PWR_MASK
#define HE_TB_PWR1011_MCS11_MIN_PWR		HE_TB_PWR1011_MCS11_MIN_PWR_MASK
#define HE_TB_PWR1011_MCS11_MAX_PWR		HE_TB_PWR1011_MCS11_MAX_PWR_MASK
#define OBSS_PD_TX_PWR_OPT_CCA_IDLE_SLOT_CNT	OBSS_PD_TX_PWR_OPT_CCA_IDLE_SLOT_CNT_MASK
#define PHYRX_DELAY_PHYRX_DELAY_VALUE		PHYRX_DELAY_PHYRX_DELAY_VALUE_MASK
#define PHYRX_DELAY_WN_SLOT_TIME		PHYRX_DELAY_WN_SLOT_TIME_MASK
#define PHYRX_DELAY_HETB_SEND_TM		PHYRX_DELAY_HETB_SEND_TM_MASK
#define AID_CMD_AID_CMD				AID_CMD_AID_CMD_MASK
#define AID_CMD_RX_AID_MONITOR			AID_CMD_RX_AID_MONITOR_MASK
#define AID_CMD_TAG_IDX				AID_CMD_TAG_IDX_MASK
#define AID_STATUS_AID_VALID			AID_STATUS_AID_VALID_MASK
#define AID_MAC_ADDR0_MAC_ADDR_31_0		AID_MAC_ADDR0_MAC_ADDR_31_0_MASK
#define AID_MAC_ADDR1_MAC_ADDR_47_32		AID_MAC_ADDR1_MAC_ADDR_47_32_MASK
#define AID_MAC_AID_AID				AID_MAC_AID_AID_MASK
#define AID_INFO0_MAC_ADDR_31_0			AID_INFO0_MAC_ADDR_31_0_MASK
#define AID_INFO1_MAC_ADDR_47_32		AID_INFO1_MAC_ADDR_47_32_MASK
#define AID_INFO1_MAC_AID			AID_INFO1_MAC_AID_MASK
#define OBSS_SR_NON_SRG_CFG_NON_SRG_THR		OBSS_SR_NON_SRG_CFG_NON_SRG_THR_MASK
#define OBSS_SR_NON_SRG_CFG_NON_SRG_ER_THR	OBSS_SR_NON_SRG_CFG_NON_SRG_ER_THR_MASK
#define OBSS_SR_NON_SRG_CFG_NON_SRG_OBSS_PD_EN	OBSS_SR_NON_SRG_CFG_NON_SRG_OBSS_PD_EN_MASK
#define OBSS_SR_NON_SRG_CFG_NON_SRG_NOBSSID_EN	OBSS_SR_NON_SRG_CFG_NON_SRG_NOBSSID_EN_MASK
#define OBSS_SR_NON_SRG_CFG_NON_SRG_AP_DIS	OBSS_SR_NON_SRG_CFG_NON_SRG_AP_DIS_MASK
#define OBSS_SR_NON_SRG_CFG_NON_SRG_TX_SRP_NON_SRG_DIS \
	OBSS_SR_NON_SRG_CFG_NON_SRG_TX_SRP_NON_SRG_DIS_MASK
#define OBSS_SR_NON_SRG_COLOR_EN0_COLOR_EN_L	OBSS_SR_NON_SRG_COLOR_EN0_COLOR_EN_L_MASK
#define OBSS_SR_NON_SRG_COLOR_EN1_COLOR_EN_H	OBSS_SR_NON_SRG_COLOR_EN1_COLOR_EN_H_MASK
#define OBSS_SR_NON_SRG_BSSID_EN0_BSSID_EN_L	OBSS_SR_NON_SRG_BSSID_EN0_BSSID_EN_L_MASK
#define OBSS_SR_NON_SRG_BSSID_EN1_BSSID_EN_H	OBSS_SR_NON_SRG_BSSID_EN1_BSSID_EN_H_MASK
#define SRG_CFG_SRG_THR				SRG_CFG_SRG_THR_MASK
#define SRG_CFG_SRG_ER_THR			SRG_CFG_SRG_ER_THR_MASK
#define SRG_CFG_SRG_EN				SRG_CFG_SRG_EN_MASK
#define SRG_MAP_COLOR0_COLOR_MAP_L		SRG_MAP_COLOR0_COLOR_MAP_L_MASK
#define SRG_MAP_COLOR1_COLOR_MAP_H		SRG_MAP_COLOR1_COLOR_MAP_H_MASK
#define SRG_MAP_BSSID0_BSSID_MAP_L		SRG_MAP_BSSID0_BSSID_MAP_L_MASK
#define SRG_MAP_BSSID1_BSSID_MAP_H		SRG_MAP_BSSID1_BSSID_MAP_H_MASK
#define SRG_SR_COLOR_EN0_COLOR_EN_L		SRG_SR_COLOR_EN0_COLOR_EN_L_MASK
#define SRG_SR_COLOR_EN1_COLOR_EN_H		SRG_SR_COLOR_EN1_COLOR_EN_H_MASK
#define SRG_SR_BSSID_EN0_BSSID_EN_L		SRG_SR_BSSID_EN0_BSSID_EN_L_MASK
#define SRG_SR_BSSID_EN1_BSSID_EN_H		SRG_SR_BSSID_EN1_BSSID_EN_H_MASK
#define SR_CFG_SR_PLCP_HEADER_EN		SR_CFG_SR_PLCP_HEADER_EN_MASK
#define SR_CFG_SR_MPDU_EN			SR_CFG_SR_MPDU_EN_MASK
#define CTS_TX_PWR_CTS_TX_PWR			CTS_TX_PWR_CTS_TX_PWR_MASK
#define OPT_RSVD_RESERVED			OPT_RSVD_RESERVED_MASK
#define OPT_RSVD1_RESERVED1			OPT_RSVD1_RESERVED1_MASK

/* VIFXXX_MASK */
#define VIF_CFG0_EN				VIF0_CFG0_EN_MASK
#define VIF_CFG0_BP				VIF0_CFG0_BP_MASK
#define VIF_CFG0_PT				VIF0_CFG0_PT_MASK
#define VIF_CFG0_TU				VIF0_CFG0_TU_MASK
#define VIF_CFG0_TOP				VIF0_CFG0_TOP_MASK
#define VIF_CFG0_TC				VIF0_CFG0_TC_MASK
#define VIF_CFG1_BASIC_HT_MCS_SET		VIF0_CFG1_BASIC_HT_MCS_SET_MASK
#define VIF_CFG1_BASIC_RATE_BITMAP		VIF0_CFG1_BASIC_RATE_BITMAP_MASK
#define VIF_RX_FILTER0_MAC_EN			VIF0_RX_FILTER0_MAC_EN_MASK
#define VIF_RX_FILTER0_BSS_EN			VIF0_RX_FILTER0_BSS_EN_MASK
#define VIF_RX_FILTER0_BFE			VIF0_RX_FILTER0_BFE_MASK
#define VIF_RX_FILTER0_AP_MODE			VIF0_RX_FILTER0_AP_MODE_MASK
#define VIF_RX_FILTER1_CTRL_FRAME_FILTER_BITMAP	VIF0_RX_FILTER1_CTRL_FRAME_FILTER_BITMAP_MASK
#define VIF_RX_FILTER1_MGNT_FRAME_FILTER_BITMAP	VIF0_RX_FILTER1_MGNT_FRAME_FILTER_BITMAP_MASK
#define VIF_MAC0_L_MAC_ADDR0_31_0		VIF0_MAC0_L_MAC_ADDR0_31_0_MASK
#define VIF_MAC0_H_MAC_ADDR0_47_36		VIF0_MAC0_H_MAC_ADDR0_47_36_MASK
#define VIF_MAC1_L_MAC_ADDR1_31_0		VIF0_MAC1_L_MAC_ADDR1_31_0_MASK
#define VIF_MAC1_H_MAC_ADDR1_47_16		VIF0_MAC1_H_MAC_ADDR1_47_16_MASK
#define VIF_BSS0_L_BSSID0_31_0			VIF0_BSS0_L_BSSID0_31_0_MASK
#define VIF_BSS0_H_BSSID0_47_16			VIF0_BSS0_H_BSSID0_47_16_MASK
#define VIF_BSS1_L_BSSID1_31_0			VIF0_BSS1_L_BSSID1_31_0_MASK
#define VIF_BSS1_H_BSSID1_47_16			VIF0_BSS1_H_BSSID1_47_16_MASK
#define VIF_AID_AID				VIF0_AID_AID_MASK
#define VIF_BFM0_HT_NG				VIF0_BFM0_HT_NG_MASK
#define VIF_BFM0_HT_CB				VIF0_BFM0_HT_CB_MASK
#define VIF_BFM0_HT_FORMAT			VIF0_BFM0_HT_FORMAT_MASK
#define VIF_BFM0_HT_MCS				VIF0_BFM0_HT_MCS_MASK
#define VIF_BFM0_VHT_NG				VIF0_BFM0_VHT_NG_MASK
#define VIF_BFM0_VHT_SU_CB			VIF0_BFM0_VHT_SU_CB_MASK
#define VIF_BFM0_VHT_MU_CB			VIF0_BFM0_VHT_MU_CB_MASK
#define VIF_BFM0_VHT_FORMAT			VIF0_BFM0_VHT_FORMAT_MASK
#define VIF_BFM0_VHT_MCS			VIF0_BFM0_VHT_MCS_MASK
#define VIF_BFM1_HE_NG				VIF0_BFM1_HE_NG_MASK
#define VIF_BFM1_HE_CB				VIF0_BFM1_HE_CB_MASK
#define VIF_BFM1_HE_FORMAT			VIF0_BFM1_HE_FORMAT_MASK
#define VIF_BFM1_HE_MCS				VIF0_BFM1_HE_MCS_MASK
#define VIF_BFM2_SNR0_LIMIT_EN			VIF0_BFM2_SNR0_LIMIT_EN_MASK
#define VIF_BFM2_SNR0_MAX			VIF0_BFM2_SNR0_MAX_MASK
#define VIF_BFM2_SNR1_LIMIT_EN			VIF0_BFM2_SNR1_LIMIT_EN_MASK
#define VIF_BFM2_SNR1_MAX			VIF0_BFM2_SNR1_MAX_MASK
#define VIF_TSF_L_TSF_31_0			VIF0_TSF_L_TSF_31_0_MASK
#define VIF_TSF_H_TSF_63_32			VIF0_TSF_H_TSF_63_32_MASK
#define VIF_TSF_OFFSET_L_TSF_OFFSET_31_0	VIF0_TSF_OFFSET_L_TSF_OFFSET_31_0_MASK
#define VIF_TSF_OFFSET_H_TSF_OFFSET_63_32	VIF0_TSF_OFFSET_H_TSF_OFFSET_63_32_MASK
#define VIF_TSF_TIME0_TSF_TIME0			VIF0_TSF_TIME0_TSF_TIME0_MASK
#define VIF_TSF_TIME1_TSF_TIME0			VIF0_TSF_TIME1_TSF_TIME0_MASK
#define VIF_TRIG_RESP_QUEUE_EN_RSVD		VIF0_TRIG_RESP_QUEUE_EN_RSVD_MASK
#define VIF_BSR_BSR_CONTENT			VIF0_BSR_BSR_CONTENT_MASK
#define VIF_MIN_MPDU_SPACING_MMSS		VIF0_MIN_MPDU_SPACING_MMSS_MASK
#define VIF_MIN_MPDU_SPACING_DEFAULT_PE_DURATION \
	VIF0_MIN_MPDU_SPACING_DEFAULT_PE_DURATION_MASK
#define VIF_MIN_MPDU_SPACING_BSS_COLOR		VIF0_MIN_MPDU_SPACING_BSS_COLOR_MASK
#define VIF_MIN_MPDU_SPACING_COLOR_EN		VIF0_MIN_MPDU_SPACING_COLOR_EN_MASK
#define VIF_MIN_MPDU_SPACING_COLOR_DIS		VIF0_MIN_MPDU_SPACING_COLOR_DIS_MASK
#define VIF_MIN_MPDU_SPACING_TRIG_EN		VIF0_MIN_MPDU_SPACING_TRIG_EN_MASK
#define VIF_MIN_MPDU_SPACING_TRIG_ASSO_UORA_EN	VIF0_MIN_MPDU_SPACING_TRIG_ASSO_UORA_EN_MASK
#define VIF_MIN_MPDU_SPACING_TRIG_UNASSO_UORA_EN \
	VIF0_MIN_MPDU_SPACING_TRIG_UNASSO_UORA_EN_MASK
#define VIF_MIN_MPDU_SPACING_TRS_EN		VIF0_MIN_MPDU_SPACING_TRS_EN_MASK
#define VIF_MIN_MPDU_SPACING_PARTIAL_BSS_COLOR_EN \
	VIF0_MIN_MPDU_SPACING_PARTIAL_BSS_COLOR_EN_MASK
#define VIF_MIN_MPDU_SPACING_BASIC_NAV_EN	VIF0_MIN_MPDU_SPACING_BASIC_NAV_EN_MASK
#define VIF_MIN_MPDU_SPACING_HETB_DATA_DIS	VIF0_MIN_MPDU_SPACING_HETB_DATA_DIS_MASK
#define VIF_MIN_MPDU_SPACING_BSRP_BSR_EN	VIF0_MIN_MPDU_SPACING_BSRP_BSR_EN_MASK
#define VIF_MIN_MPDU_SPACING_BSRP_MORE_QOSNULL_EN \
	VIF0_MIN_MPDU_SPACING_BSRP_MORE_QOSNULL_EN_MASK
#define VIF_QNULL_CTRL_AC0_SEQUNECE_CTRL	VIF0_QNULL_CTRL_AC0_SEQUNECE_CTRL_MASK
#define VIF_QNULL_CTRL_AC0_QOS_CTRL		VIF0_QNULL_CTRL_AC0_QOS_CTRL_MASK
#define VIF_QNULL_CTRL_AC1_SEQUNECE_CTRL	VIF0_QNULL_CTRL_AC1_SEQUNECE_CTRL_MASK
#define VIF_QNULL_CTRL_AC1_QOS_CTRL		VIF0_QNULL_CTRL_AC1_QOS_CTRL_MASK
#define VIF_QNULL_CTRL_AC2_SEQUNECE_CTRL	VIF0_QNULL_CTRL_AC2_SEQUNECE_CTRL_MASK
#define VIF_QNULL_CTRL_AC2_QOS_CTRL		VIF0_QNULL_CTRL_AC2_QOS_CTRL_MASK
#define VIF_QNULL_CTRL_AC3_SEQUNECE_CTRL	VIF0_QNULL_CTRL_AC3_SEQUNECE_CTRL_MASK
#define VIF_QNULL_CTRL_AC3_QOS_CTRL		VIF0_QNULL_CTRL_AC3_QOS_CTRL_MASK
#define VIF_UORA_CFG_OBO_INIT_VALUE		VIF0_UORA_CFG_OBO_INIT_VALUE_MASK
#define VIF_UORA_CFG_SW_OBO_VALUE_EN		VIF0_UORA_CFG_SW_OBO_VALUE_EN_MASK
#define VIF_UORA_CFG_CW_INIT			VIF0_UORA_CFG_CW_INIT_MASK
#define VIF_UORA_CFG_CW_MIN			VIF0_UORA_CFG_CW_MIN_MASK
#define VIF_UORA_CFG_CW_MAX			VIF0_UORA_CFG_CW_MAX_MASK
#define VIF_UORA_CFG_STATUS_OBO_INIT_VALUE	VIF0_UORA_CFG_STATUS_OBO_INIT_VALUE_MASK
#define VIF_UORA_CFG_STATUS_SW_OBO_VALUE_EN	VIF0_UORA_CFG_STATUS_SW_OBO_VALUE_EN_MASK
#define VIF_UORA_CFG_STATUS_OBO_VALUE		VIF0_UORA_CFG_STATUS_OBO_VALUE_MASK
#define VIF_UORA_CFG_STATUS_CW_INIT		VIF0_UORA_CFG_STATUS_CW_INIT_MASK
#define VIF_UORA_CFG_STATUS_CW_MIN		VIF0_UORA_CFG_STATUS_CW_MIN_MASK
#define VIF_UORA_CFG_STATUS_CW_MAX		VIF0_UORA_CFG_STATUS_CW_MAX_MASK
#define VIF_UORA_CFG_STATUS_CURR_CW		VIF0_UORA_CFG_STATUS_CURR_CW_MASK
#define VIF_UORA_CFG1_CHANNEL_EN		VIF0_UORA_CFG1_CHANNEL_EN_MASK
#define VIF_UORA_CFG1_LTF_MODE_EN		VIF0_UORA_CFG1_LTF_MODE_EN_MASK
#define VIF_UORA_CFG1_80P80_MODE		VIF0_UORA_CFG1_80P80_MODE_MASK
#define VIF_UORA_CFG1_PRI_CH			VIF0_UORA_CFG1_PRI_CH_MASK
#define VIF_NON_SRG_OBSS_PD_MIN_VALUE		VIF0_NON_SRG_OBSS_PD_MIN_VALUE_MASK
#define VIF_NON_SRG_OBSS_PD_MAX_VALUE		VIF0_NON_SRG_OBSS_PD_MAX_VALUE_MASK
#define VIF_SRG_OBSS_PD_MIN_VALUE		VIF0_SRG_OBSS_PD_MIN_VALUE_MASK
#define VIF_SRG_OBSS_PD_MAX_VALUE		VIF0_SRG_OBSS_PD_MAX_VALUE_MASK
#define VIF_TX_PWR_REF_PWR			VIF0_TX_PWR_REF_PWR_MASK
#define VIF_TX_PWR_HW_PWR			VIF0_TX_PWR_HW_PWR_MASK
#define VIF_CFG_INFO_QUE_CONTENTION_EN		VIF0_CFG_INFO_QUE_CONTENTION_EN_MASK
#define VIF_CFG_INFO_NAV_SWITCH			VIF0_CFG_INFO_NAV_SWITCH_MASK
#define VIF_CFG_INFO_QOS_NULL_UPH_EN		VIF0_CFG_INFO_QOS_NULL_UPH_EN_MASK
#define VIF_CFG_INFO_QUEUE_UPH_EN		VIF0_CFG_INFO_QUEUE_UPH_EN_MASK
#define VIF_CFG_INFO_QUEUE_HTC_APPEND_EN	VIF0_CFG_INFO_QUEUE_HTC_APPEND_EN_MASK
#define VIF_CFG_INFO_CTS_NAV_CHK_DIS		VIF0_CFG_INFO_CTS_NAV_CHK_DIS_MASK
#define VIF_CFG_INFO_TRIG_NAV_ASSO_CHK_DIS	VIF0_CFG_INFO_TRIG_NAV_ASSO_CHK_DIS_MASK
#define VIF_CFG_INFO_TRIG_NAV_UNASSO_CHK_DIS	VIF0_CFG_INFO_TRIG_NAV_UNASSO_CHK_DIS_MASK
#define VIF_CFG_INFO_TRIG_CS_ASSO_CHK_DIS	VIF0_CFG_INFO_TRIG_CS_ASSO_CHK_DIS_MASK
#define VIF_CFG_INFO_TRIG_CS_UNASSO_CHK_DIS	VIF0_CFG_INFO_TRIG_CS_UNASSO_CHK_DIS_MASK
#define VIF_CFG_INFO_SUBCHANNEL_20_MAPPING	VIF0_CFG_INFO_SUBCHANNEL_20_MAPPING_MASK
#define VIF_CFG_INFO_OBSS_NARROW_BW_RU_IN_OFDMA_LOST \
	VIF0_CFG_INFO_OBSS_NARROW_BW_RU_IN_OFDMA_LOST_MASK
#define VIF_CFG_INFO_DFS_50_100_CON		VIF0_CFG_INFO_DFS_50_100_CON_MASK
#define VIF_CFG_INFO_QUEUE_HTC_CONVERT_EN	VIF0_CFG_INFO_QUEUE_HTC_CONVERT_EN_MASK
#define VIF_CFG_INFO_QOSNULL_SEQ_INC_DIS	VIF0_CFG_INFO_QOSNULL_SEQ_INC_DIS_MASK
#define VIF_CFG_INFO_COLOR_INBSS		VIF0_CFG_INFO_COLOR_INBSS_MASK
#define VIF_CFG_INFO_OP_MODE			VIF0_CFG_INFO_OP_MODE_MASK
#define VIF_MBSSID_MASK_L32_MBSSID_MASK_31_0	VIF0_MBSSID_MASK_L32_MBSSID_MASK_31_0_MASK
#define VIF_MBSSID_MASK_H16_MBSSID_MASK_47_32	VIF0_MBSSID_MASK_H16_MBSSID_MASK_47_32_MASK
#define VIF_TRIG_RESOURCE_THR_EXP		VIF0_TRIG_RESOURCE_THR_EXP_MASK
#define VIF_TRIG_RESOURCE_EXP			VIF0_TRIG_RESOURCE_EXP_MASK
#define VIF_TRIG_INTF_ASLEEP			VIF0_TRIG_INTF_ASLEEP_MASK
#define VIF_TRIG_O_80_BUSY			VIF0_TRIG_O_80_BUSY_MASK
#define VIF_TRIG_O_80_LOC			VIF0_TRIG_O_80_LOC_MASK
#define VIF_TRIG_BQR_CH_MASK			VIF0_TRIG_BQR_CH_MASK_MASK
#define VIF_TRIG_BQRP_CH20_MAPPING		VIF0_TRIG_BQRP_CH20_MAPPING_MASK
#define VIF_ESP_SP_CHECK_EN			VIF0_ESP_SP_CHECK_EN_MASK
#define VIF_ESP_RSVD_ACK_BA_MORE_DATA_CHK	VIF0_ESP_RSVD_ACK_BA_MORE_DATA_CHK_MASK
#define VIF_ESP_CTRL_MORE_DATA_CHK		VIF0_ESP_CTRL_MORE_DATA_CHK_MASK
#define VIF_ESP_STATUS_SP_CHECK_EN		VIF0_ESP_STATUS_SP_CHECK_EN_MASK
#define VIF_ESP_STATUS_RSVD_ACK_BA_MORE_DATA_CHK \
	VIF0_ESP_STATUS_RSVD_ACK_BA_MORE_DATA_CHK_MASK
#define VIF_ESP_STATUS_CTRL_MORE_DATA_CHK	VIF0_ESP_STATUS_CTRL_MORE_DATA_CHK_MASK
#define VIF_ESP_STATUS_FRAME_TYPE		VIF0_ESP_STATUS_FRAME_TYPE_MASK
#define VIF_ESP_STATUS_FRAME_SUBTYPE		VIF0_ESP_STATUS_FRAME_SUBTYPE_MASK
#define VIF_ESP_STATUS_MORE_DATE		VIF0_ESP_STATUS_MORE_DATE_MASK
#define VIF_ESP_STATUS_QOS_EOSP			VIF0_ESP_STATUS_QOS_EOSP_MASK
#define VIF_HTC_OMI_HETB_QOSNULL_OMI_EN		VIF0_HTC_OMI_HETB_QOSNULL_OMI_EN_MASK
#define VIF_HTC_OMI_HETB_DATA_OMI_EN		VIF0_HTC_OMI_HETB_DATA_OMI_EN_MASK
#define VIF_HTC_OMI_HE_OMI			VIF0_HTC_OMI_HE_OMI_MASK
#define VIF_BASIC_TRIG_USER_INFO_RX_BASIC_TRIGGER_USER_VIF0 \
	VIF0_BASIC_TRIG_USER_INFO_RX_BASIC_TRIGGER_USER_VIF0_MASK
#define VIF_BASIC_TRIG_PSDU_LEN_RX_BASIC_TRIGGER_HETB_PSDU_LEN_VIF0 \
	VIF0_BASIC_TRIG_PSDU_LEN_RX_BASIC_TRIGGER_HETB_PSDU_LEN_VIF0_MASK
#define VIF_RSVD_RESERVED			VIF0_RSVD_RESERVED_MASK

/* TX_QUEN_XXX_MASK */
#define TX_QUEN_CMD_CMD_EDCA			TX_QUEN_CMD0_CMD_EDCA_MASK
#define TX_QUEN_CMD_AIFSN			TX_QUEN_CMD0_AIFSN_MASK
#define TX_QUEN_CMD_BOFF_COUNT			TX_QUEN_CMD0_BOFF_COUNT_MASK
#define TX_QUEN_TXOP_CMD			TX_QUEN_TXOP0_CMD_MASK
#define TX_QUEN_TXOP_CFE_BS_TA			TX_QUEN_TXOP0_CFE_BS_TA_MASK
#define TX_QUEN_TXOP_TX_PRI			TX_QUEN_TXOP0_TX_PRI_MASK
#define TX_QUEN_STATE_TX_QUEUE_STATE		TX_QUEN_STATE0_TX_QUEUE_STATE_MASK
#define TX_QUEN_TIMER_TXOP_TIMER		TX_QUEN_TIMER0_TXOP_TIMER_MASK
/* Add unused define*/
#ifndef VIF_CFG0_SLOT_TIME
#define VIF_CFG0_SLOT_TIME			0xFF0000
#endif

#ifndef DEV_CFG_SLOT_TIME
#define DEV_CFG_SLOT_TIME			0x01
#endif

#ifndef RX_BUF_CFG_DMA_EN
#define RX_BUF_CFG_DMA_EN			0x01
#endif

/* TX registers */
#define REG_TX_STRIDE				(REG_TX_QUEN_CMD1 - REG_TX_QUEN_CMD0)
#define TXQ(n)					((n) * REG_TX_STRIDE)

#define MAC_REG_TX_QUEN_CMD(n)			(TXQ(n) + MAC_REG_TX_QUEN_CMD0)
#define MAC_REG_TX_QUEN_TXOP(n)			(TXQ(n) + MAC_REG_TX_QUEN_TXOP0)
#define MAC_REG_TX_QUEN_STATE(n)		(TXQ(n) + MAC_REG_TX_QUEN_STATE0)
#define MAC_REG_TX_QUEN_TIMER(n)		(TXQ(n) + MAC_REG_TX_QUEN_TIMER0)
#define MAC_REG_VIF_QNULL_CTRL_AC(v, ac)	(MAC_REG_VIF_QNULL_CTRL_AC0(v) + (ac) * 4)

/* Bit fields */
#define VIF_QNULL_CTRL_AC_SEQUNECE_CTRL		VIF0_QNULL_CTRL_AC0_SEQUNECE_CTRL_MASK
#define VIF_QNULL_CTRL_AC_QOS_CTRL		VIF0_QNULL_CTRL_AC0_QOS_CTRL_MASK

#define TX_QUEN_CMD_EDCA			BIT(0)
#define TX_QUEN_CMD_AID_MATCHED_QUEUE_EN	BIT(1)
#define TX_QUEN_CMD_ASSO_UORA_QUEUE_EN		BIT(2)
#define TX_QUEN_CMD_UNASSO_UORA_QUEUE_EN	BIT(3)
#define TX_QUEN_CMD_HE_TB_RESP_MASK		(TX_QUEN_CMD_CMD_EDCA & ~TX_QUEN_CMD_EDCA)

/* Debug registers */
#define REG_DBG_RX_PPDU_CNT(n)		(REG_DBG_RX_PPDU_CNT0 + 0x04 * (n))

/**
 * PHY registers supplimentary definitions
 *
 * @PHY_BASE(i): Base address of PHY i
 * @PHY_MODEM(reg): Modem register offset
 * @PHY_RFIF(reg): RF interface register offset
 */
#define PHY_MODEM_OFFSET	0x0
#define PHY_RFI_OFFSET          0x8000

#define PHY_BASE(i)		((i) * 0x4000)
#define PHY_MODEM(reg)		(reg)
#define PHY_RFIF(reg)		(PHY_RFI_OFFSET + (reg))

/*
 * PHY VIF (PHY context) banked registers
 *
 * To support fast VIF (or channel) switching, the following registers are
 * banked. Bank selection is done by setting FW_SEL_VIF (0: bank 0, 1: bank 1)
 *
 * #define FW_OPBW_PRICH(v)	(FW_OPBW_PRICH_VINTF0 + PHY_CTX(v))
 */

#define PHY_VINTF_STRIDE        (FW_OPBW_PRICH_VINTF1 - FW_OPBW_PRICH_VINTF0)
#define PHY_CTX(v)		((v) * PHY_VINTF_STRIDE)

#define FW_OPBW_PRICH_VINTF(v)		(FW_OPBW_PRICH_VINTF0 + PHY_CTX(v))
#define FW_MIMO_EN_VINTF(v)		(FW_MIMO_EN_VINTF0 + PHY_CTX(v))
#define FW_SIGEXT_VINTF(v)		(FW_SIGEXT_VINTF0 + PHY_CTX(v))
#define FW_VHT_CONFIG0_VINTF(v)		(FW_VHT_CONFIG0_VINTF0 + PHY_CTX(v))
#define FW_VHT_CONFIG1_VINTF(v)		(FW_VHT_CONFIG1_VINTF0 + PHY_CTX(v))
#define FW_VHT_CONFIG2_VINTF(v)		(FW_VHT_CONFIG2_VINTF0 + PHY_CTX(v))
#define FW_VHT_CONFIG3_VINTF(v)		(FW_VHT_CONFIG3_VINTF0 + PHY_CTX(v))
#define FW_VHT_CONFIG4_VINTF(v)		(FW_VHT_CONFIG4_VINTF0 + PHY_CTX(v))
#define FW_VHT_CONFIG5_VINTF(v)		(FW_VHT_CONFIG5_VINTF0 + PHY_CTX(v))
#define FW_VHT_CONFIG6_VINTF(v)		(FW_VHT_CONFIG6_VINTF0 + PHY_CTX(v))
#define FW_VHT_CONFIG7_VINTF(v)		(FW_VHT_CONFIG7_VINTF0 + PHY_CTX(v))
#define FW_VHT_CONFIG8_VINTF(v)		(FW_VHT_CONFIG8_VINTF0 + PHY_CTX(v))
#define FW_HE_CONFIG1_VINTF(v)		(FW_HE_CONFIG1_VINTF0 + PHY_CTX(v))
#define FW_HE_CONFIG2_VINTF(v)		(FW_HE_CONFIG2_VINTF0 + PHY_CTX(v))
#define FW_11B_CSFO_CONV_VINTF(v)	(FW_11B_CSFO_CONV_VINTF0 + PHY_CTX(v))
#define FW_11B_PLLADJ_VINTF(v)		(FW_11B_PLLADJ_VINTF0 + PHY_CTX(v))
#define FW_11B_TX_PLLADJ_VINTF(v)	(FW_11B_TX_PLLADJ_VINTF0 + PHY_CTX(v))
/**
 * RF physical address
 */

#define RF_BASE_START		0x0

/**
 * RF interface
 */

/* RFI_CTRLSIG_CFG_XX fields */
#define RFI_CTRLSIG_CFG(x)	PHY_RFIF(FW_RFI_CTRLSIG_CFG_00 + ((x) * 4))

#define RFI_CTRLSIG_CFG_EN	BIT(21)
#define RFI_CTRLSIG_CFG_POL	BIT(20)
#define RFI_CTRLSIG_CFG_SSEL	GENMASK(19, 18)
#define RFI_CTRLSIG_CFG_ESEL	GENMASK(17, 16)
#define RFI_CTRLSIG_CFG_D0	GENMASK(15,  8)
#define RFI_CTRLSIG_CFG_D1	GENMASK(15,  8)

#define TXEN			0x0
#define TXVALID			0x1
#define RXEN			0x2
#define ALWAYS			0x3

#define HIGH			0x0
#define LOW			0x1

#define RFCTRL(en, pol, on, off, d0, d1)	  \
	(((en) << 21) |				  \
	 ((pol) << 20) |			  \
	 (((on) & 0x3) << 18) |			  \
	 (((off) & 0x3) << 16) |		  \
	 (((d0) & 0xff) << 8) |			  \
	 ((d1) & 0xff))

#endif /* __SCM80211_REGS_H__ */
