/* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause */
/*
 * Copyright (c) 2020-2023 Senscomm Semiconductor Co., Ltd.
 *
 */
#ifndef __SCM80211_RF_H__
#define __SCM80211_RF_H__

#include <linux/platform_device.h>

#include "phy.h"

/**
 * DOC: scm2020 RF board
 *
 * scm2020 has 2 RF chains
 */

struct scm2020 {
	struct scm80211_radio *radio;
	int id;			/* board id. I know it's ugly */
	char path;
	struct ieee80211_hw *hw;
};

/**
 * struct scm80211_rf_devce - external rf device
 * collection of radio chains
 *
 * @id: board number
 */
struct scm80211_rf_device {
	struct platform_device		pdev;
	int				board_id;
	struct scm80211_radio		*radios;
	int				n_chain;
	struct ieee80211_hw		*hw;
	struct completion		completion;
};

/*#define to_rf_device(pdev) dev_get_platdata(&pdev->dev)*/
#define to_rf_device(pdev) container_of(pdev, struct scm80211_rf_device, pdev)

struct scm80211_rf_driver {
	struct platform_driver platform_driver;
};

#define to_rf_driver(pdriver) container_of(pdriver, struct scm80211_rf_driver, driver)

struct scm80211_rf_module {
	char *name;
	int (*init)(void);
	void (*exit)(void);
};

int scm80211_init_rf_modules(void);
void scm80211_exit_rf_modules(void);

struct scm80211_rf_module *phy_find_rf(struct ieee80211_hw *hw, char *name);
struct scm80211_rf_device *scm80211_alloc_rf_device(struct ieee80211_hw *hw, const char *name,
						    int board_id);
int phy_attach_rf(struct ieee80211_hw *hw, struct scm80211_rf_device *rf);

void scm80211_wait_for_rf_ready(struct scm80211_rf_device *rf);
void scm80211_notify_rf_ready(struct scm80211_rf_device *rf);

#define __RF__							\
	__attribute((__used__))					\
	__attribute((__section__("scm80211_rf_module")))	\
	__attribute((aligned(sizeof(void *))))			\

#define RF_MODULE(x)						\
static struct scm80211_rf_module rf_module_##x __RF__ = {	\
	.name = #x,						\
	.init = x##_init,						\
	.exit = x##_exit,						\
}

extern struct scm80211_rf_system rf_table_start, rf_table_end;
extern struct scm80211_rf_module rf_module_start, rf_module_end;

#endif /* __SCM80211_RF_H__ */
