/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Portions of this file
 * Copyright(c) 2016-2017 Intel Deutschland GmbH
 * Copyright (C) 2018 - 2019 Intel Corporation
 */

#if !defined(__SCM80211_DRIVER_TRACE) || defined(TRACE_HEADER_MULTI_READ)
#include <linux/tracepoint.h>
#include <net/mac80211.h>

#include "core.h"
#include "util.h"

#undef TRACE_SYSTEM
#define TRACE_SYSTEM scm80211

/* struct ieee80211_hw */
#define MAXNAME		32
#define HW_ENTRY	__array(char, wiphy_name, 32)
#define HW_ASSIGN	strscpy(__entry->wiphy_name, wiphy_name(hw->wiphy), MAXNAME)
#define HW_PR_FMT	"%s"
#define HW_PR_ARG	__entry->wiphy_name

/* struct ieee80211_vif */
#define VIF_ENTRY	__field(enum nl80211_iftype, vif_type) __field(void *, vif)	\
			__field(bool, p2p)						\
			__array(char, vif_name, 32)

#define VIF_ASSIGN	__entry->vif_type = vif->type; __entry->vif = vif;		\
			__entry->p2p = vif->p2p;					\
			(ieee80211_vif_name(vif)) ?					\
				strcpy(__entry->vif_name, ieee80211_vif_name(vif)) :	\
				memset(__entry->vif_name, 0, 32);

#define VIF_PR_FMT	" vif:%s(%d%s)"
#define VIF_PR_ARG	__entry->vif_name, __entry->vif_type, __entry->p2p ? "/p2p" : ""

/* struct ieee80211_sta */
#define STA_ENTRY	__array(char, sta_addr, ETH_ALEN)
#define STA_ASSIGN(sta)	(sta ? memcpy(__entry->sta_addr, sta->addr, ETH_ALEN) : memset(__entry->sta_addr, 0, ETH_ALEN))
#define STA_NAMED_ASSIGN(s)	memcpy(__entry->sta_addr, (s)->addr, ETH_ALEN)
#define STA_PR_FMT	" sta:%pM"
#define STA_PR_ARG	__entry->sta_addr

/* struct ieee80211_sta_rates */
#define RATES_ENTRY	__field(s8, rate0_idx) __field(u16, rate0_flags)		\
			__field(s8, rate1_idx) __field(u16, rate1_flags)		\
			__field(s8, rate2_idx) __field(u16, rate2_flags)

#define RATES_ASSIGN	__entry->rate0_idx = rates->rate[0].idx;			\
			__entry->rate1_idx = rates->rate[1].idx;			\
			__entry->rate2_idx = rates->rate[2].idx;			\
			__entry->rate0_flags = rates->rate[0].flags;			\
			__entry->rate1_flags = rates->rate[1].flags;			\
			__entry->rate2_flags = rates->rate[2].flags;

#define RATES_PR_FMT	" rates:[%d, %04x], [%d, %04x], [%d, %04x]"
#define RATES_PR_ARG	__entry->rate0_idx, __entry->rate0_flags,			\
			__entry->rate1_idx, __entry->rate1_flags,			\
			__entry->rate2_idx, __entry->rate2_flags

/* struct scm80211_chanctx */
#define CHANDEF_ENTRY	__field(u32, control_freq)					\
			__field(u32, chan_width)					\
			__field(u32, center_freq1)					\
			__field(u32, center_freq2)
#define CHANDEF_ASSIGN(c)							\
			__entry->control_freq = (c) ? ((c)->chan ? (c)->chan->center_freq : 0) : 0;	\
			__entry->chan_width = (c) ? (c)->width : 0;			\
			__entry->center_freq1 = (c) ? (c)->center_freq1 : 0;		\
			__entry->center_freq2 = (c) ? (c)->center_freq2 : 0;
#define CHANDEF_PR_FMT	" control:%d MHz width:%d center: %d/%d MHz"
#define CHANDEF_PR_ARG	__entry->control_freq, __entry->chan_width,	\
			__entry->center_freq1, __entry->center_freq2

#define MIN_CHANDEF_ENTRY								\
			__field(u32, min_control_freq)					\
			__field(u32, min_chan_width)					\
			__field(u32, min_center_freq1)					\
			__field(u32, min_center_freq2)

#define MIN_CHANDEF_ASSIGN(c)								\
			__entry->min_control_freq = (c)->chan ? (c)->chan->center_freq : 0;	\
			__entry->min_chan_width = (c)->width;				\
			__entry->min_center_freq1 = (c)->center_freq1;			\
			__entry->min_center_freq2 = (c)->center_freq2;
#define MIN_CHANDEF_PR_FMT	" min_control:%d MHz min_width:%d min_center: %d/%d MHz"
#define MIN_CHANDEF_PR_ARG	__entry->min_control_freq, __entry->min_chan_width,	\
			__entry->min_center_freq1, __entry->min_center_freq2

#define CHANCTX_ENTRY	CHANDEF_ENTRY							\
			__field(u8, rx_chains_static)					\
			__field(u8, rx_chains_dynamic)					\
			__field(int, id)
#define CHANCTX_ASSIGN	CHANDEF_ASSIGN(&chanctx->ctx.def)				\
			__entry->rx_chains_static = chanctx->ctx.rx_chains_static;	\
			__entry->rx_chains_dynamic = chanctx->ctx.rx_chains_dynamic; \
			__entry->id = chanctx->id

#define CHANCTX_PR_FMT	" chanctx:%d("CHANDEF_PR_FMT " chains:%d/%d)"
#define CHANCTX_PR_ARG	__entry->id, CHANDEF_PR_ARG,					\
			__entry->rx_chains_static, __entry->rx_chains_dynamic

/* struct scm80211_txq */
#define TXQ_ENTRY	STA_ENTRY							\
			__field(u8, tid)						\
			__field(u8, ac)						\
			__field(u8, hw_queue)

#define TXQ_ASSIGN	STA_ASSIGN(txq->txq.sta);					\
			__entry->tid = txq->txq.tid;					\
			__entry->ac = txq->txq.ac;					\
			__entry->hw_queue = txq->hw_queue;
#define TXQ_PR_FMT	STA_PR_FMT "(tid:%d hwqueue:%d)"
#define TXQ_PR_ARG	STA_PR_ARG, __entry->tid, __entry->hw_queue

/* struct scm80211_hwq */
#define HWQ_ENTRY	__field(u8, id)							\

#define HWQ_ASSIGN	__entry->id = hwq->id;						\

#define HWQ_PR_FMT	" hwq:%d"
#define HWQ_PR_ARG	__entry->id

#define TXOP_ENTRY	__field(u8, hwq_id)						\
			__field(u8, txop_id)

#define TXOP_ASSIGN	__entry->hwq_id = txop->hwq->id;				\
			__entry->txop_id = txop->id;

#define TXOP_PR_FMT	"hwq:%d.%d"
#define TXOP_PR_ARG	__entry->hwq_id, __entry->txop_id

#define KEY_ENTRY	__field(u32, cipher)						\
			__field(u8, hw_key_idx)						\
			__field(u8, flags)						\
			__field(s8, keyidx)
#define KEY_ASSIGN(k)	__entry->cipher = (k)->cipher;					\
			__entry->flags = (k)->flags;					\
			__entry->keyidx = (k)->keyidx;					\
			__entry->hw_key_idx = (k)->hw_key_idx;
#define KEY_PR_FMT	" cipher:0x%x, flags=%#x, keyidx=%d, hw_key_idx=%d"
#define KEY_PR_ARG	__entry->cipher, __entry->flags, __entry->keyidx, __entry->hw_key_idx

#define AMPDU_ACTION_ENTRY	__field(enum ieee80211_ampdu_mlme_action,		\
					ieee80211_ampdu_mlme_action)			\
				STA_ENTRY						\
				__field(u16, tid)					\
				__field(u16, ssn)					\
				__field(u16, buf_size)					\
				__field(bool, amsdu)					\
				__field(u16, timeout)					\
				__field(u16, action)
#define AMPDU_ACTION_ASSIGN	STA_NAMED_ASSIGN(params->sta);				\
				__entry->tid = params->tid;				\
				__entry->ssn = params->ssn;				\
				__entry->buf_size = params->buf_size;			\
				__entry->amsdu = params->amsdu;				\
				__entry->timeout = params->timeout;			\
				__entry->action = params->action;
#define AMPDU_ACTION_PR_FMT	STA_PR_FMT " tid %d, ssn %d, buf_size %u, amsdu %d, timeout %d action %d"
#define AMPDU_ACTION_PR_ARG	STA_PR_ARG, __entry->tid, __entry->ssn,			\
				__entry->buf_size, __entry->amsdu, __entry->timeout,	\
				__entry->action

/* CHANCTX */

DECLARE_EVENT_CLASS(scm80211_chanctx,
		    TP_PROTO(struct ieee80211_hw *hw, struct scm80211_chanctx *chanctx),
		    TP_ARGS(hw, chanctx),
		    TP_STRUCT__entry(HW_ENTRY
				     CHANCTX_ENTRY),
		    TP_fast_assign(HW_ASSIGN;
				   CHANCTX_ASSIGN;),
		    TP_printk(HW_PR_FMT CHANCTX_PR_FMT, HW_PR_ARG, CHANCTX_PR_ARG)
);

DEFINE_EVENT(scm80211_chanctx, scm80211_start_chanctx,
	     TP_PROTO(struct ieee80211_hw *hw, struct scm80211_chanctx *chanctx),
	     TP_ARGS(hw, chanctx)
);

DEFINE_EVENT(scm80211_chanctx, scm80211_stop_chanctx,
	     TP_PROTO(struct ieee80211_hw *hw, struct scm80211_chanctx *chanctx),
	     TP_ARGS(hw, chanctx)
);

DEFINE_EVENT(scm80211_chanctx, scm80211_switch_chanctx,
	     TP_PROTO(struct ieee80211_hw *hw, struct scm80211_chanctx *chanctx),
	     TP_ARGS(hw, chanctx)
);

TRACE_EVENT(scm80211_vif_update_chanctx,
	    TP_PROTO(struct ieee80211_hw *hw, struct ieee80211_vif *vif,
		     struct scm80211_chanctx *chanctx, unsigned long changes),
	    TP_ARGS(hw, vif, chanctx, changes),
	    TP_STRUCT__entry(HW_ENTRY VIF_ENTRY CHANCTX_ENTRY __field(unsigned long, changes)
	    ),
	    TP_fast_assign(HW_ASSIGN;
			   VIF_ASSIGN;
			   CHANCTX_ASSIGN;
			   __entry->changes = changes;),
	    TP_printk(HW_PR_FMT VIF_PR_FMT CHANCTX_PR_FMT " changes=%08lx",
		      HW_PR_ARG, VIF_PR_ARG, CHANCTX_PR_ARG, __entry->changes)
);

DECLARE_EVENT_CLASS(scm80211_chanctx_update,
		    TP_PROTO(struct ieee80211_hw *hw, struct scm80211_chanctx *chanctx,
			     unsigned long changes),
		    TP_ARGS(hw, chanctx, changes),
		    TP_STRUCT__entry(HW_ENTRY CHANCTX_ENTRY __field(unsigned long, changes)
		    ),
		    TP_fast_assign(HW_ASSIGN;
				   CHANCTX_ASSIGN;
				   __entry->changes = changes;),
		    TP_printk(HW_PR_FMT CHANCTX_PR_FMT " changes=%08lx (%s)",
			      HW_PR_ARG, CHANCTX_PR_ARG, __entry->changes,
			      __print_flags(__entry->changes, ",", {
					    BIT(PHY_CHANNEL), "CHANNEL"}, {
					    BIT(PHY_ACTIVE_RXCHAIN_SET), "RXCHAIN"}, {
					    BIT(PHY_ANT_CONFIG), "ANT_CONFIG"}, {
					    BIT(PHY_RSSI_THRESHOLD), "RSSI_THRESHOLD"}, {
					    BIT(PHY_LISTEN_TO_GID00), "LISTEN_TO_GID00"}, {
					    BIT(PHY_LISTEN_TO_GID63), "LISTEN_TO_GID63"}, {
					    BIT(PHY_PARTIAL_AID_LIST_GID00),
					    "PARTIAL_AID_LIST_GID00"}, {
					    BIT(PHY_PARTIAL_AID_LIST_GID63),
					    "PARTIAL_AID_LIST_GID63"}, {
					    BIT(PHY_VHT_SU_RX_FILTER_EN), "VHT_SU_RX_FILTER_EN"}, {
					    BIT(PHY_GROUP_ID_MANAGEMENT), "GROUP_ID_MANAGEMENT"}, {
					    BIT(PHY_BSS_COLOR), "BSS_COLOR"}, {
					    BIT(PHY_STA_ID_INDIVIDUAL), "STA_ID_INDIVIDUAL"}, {
					    BIT(PHY_STA_ID_GROUP), "STA_ID_GROUP"}, {
					    BIT(PHY_BSS_COLOR_FILTER_EN), "BSS_COLOR_FILTER_EN"}, {
					    BIT(PHY_HE_DIR), "HE_DIR"})

		    )
);

DEFINE_EVENT(scm80211_chanctx_update, phy_config,
	     TP_PROTO(struct ieee80211_hw *hw, struct scm80211_chanctx *chanctx,
		      unsigned long changes), TP_ARGS(hw, chanctx, changes)
);

/* TXQ */
DECLARE_EVENT_CLASS(scm8021_txq,
		    TP_PROTO(struct ieee80211_hw *hw, struct scm80211_chanctx *chanctx),
		    TP_ARGS(hw, chanctx),
		    TP_STRUCT__entry(HW_ENTRY
				     CHANCTX_ENTRY),
		    TP_fast_assign(HW_ASSIGN;
				   CHANCTX_ASSIGN;),
		    TP_printk(HW_PR_FMT CHANCTX_PR_FMT, HW_PR_ARG, CHANCTX_PR_ARG)
);

/* HWQ */
DECLARE_EVENT_CLASS(scm8021_hwq,
		    TP_PROTO(struct scm80211_hwq *hwq),
		    TP_ARGS(hwq),
		    TP_STRUCT__entry(HWQ_ENTRY
				     TXQ_ENTRY),
		    TP_fast_assign(HWQ_ASSIGN;
				   TXQ_ENTRY;),
		    TP_printk(HWQ_PR_FMT TXQ_PR_FMT, HWQ_PR_ARG, TXQ_PR_ARG)
);

DECLARE_EVENT_CLASS(scm80211_hwq,
		    TP_PROTO(struct ieee80211_hw *hw, struct scm80211_hwq *hwq,
			     struct scm80211_txq *txq),
		    TP_ARGS(hw, hwq, txq),
		    TP_STRUCT__entry(HW_ENTRY
				     HWQ_ENTRY
				     TXQ_ENTRY),
		    TP_fast_assign(HW_ASSIGN;
				   HWQ_ASSIGN;
				   TXQ_ASSIGN;),
		    TP_printk(HW_PR_FMT HWQ_PR_FMT TXQ_PR_FMT, HW_PR_ARG, HWQ_PR_ARG, TXQ_PR_ARG)
);

DEFINE_EVENT(scm80211_hwq, scm80211_submit_txq,
	     TP_PROTO(struct ieee80211_hw *hw, struct scm80211_hwq *hwq,
		      struct scm80211_txq *txq), TP_ARGS(hw, hwq, txq)
);

DEFINE_EVENT(scm80211_hwq, scm80211_complete_txq,
	     TP_PROTO(struct ieee80211_hw *hw, struct scm80211_hwq *hwq,
		      struct scm80211_txq *txq), TP_ARGS(hw, hwq, txq)
);

TRACE_EVENT(scm80211_complete_txop,
	    TP_PROTO(struct ieee80211_hw *hw, struct scm80211_hwq *hwq,
		     struct scm80211_txq *txq, bool discard, u32 cmd, u32 result),
	    TP_ARGS(hw, hwq, txq, discard, cmd, result),
	    TP_STRUCT__entry(HW_ENTRY HWQ_ENTRY TXQ_ENTRY __field(bool, discard)
			     __field(u32, cmd)
			     __field(u32, result)
	    ),
	    TP_fast_assign(HW_ASSIGN;
			   HWQ_ASSIGN;
			   TXQ_ASSIGN;
			   __entry->discard = discard;
			   __entry->cmd = cmd;
			   __entry->result = result;),
	    TP_printk(HW_PR_FMT HWQ_PR_FMT TXQ_PR_FMT " discard:%d cmd:%08x status:%08x",
		      HW_PR_ARG, HWQ_PR_ARG, TXQ_PR_ARG,
		      __entry->discard, __entry->cmd, __entry->result)
);

TRACE_EVENT(scm80211_tx_result_sifs_burst,
	    TP_PROTO(struct ieee80211_hw *hw, struct scm80211_hwq *hwq,
		     struct scm80211_txq *txq, int sn, bool sent,
		     bool acked),
	    TP_ARGS(hw, hwq, txq, sn, sent, acked),
	    TP_STRUCT__entry(HW_ENTRY HWQ_ENTRY TXQ_ENTRY __field(int, sn)
			     __field(bool, sent)
			     __field(bool, acked)
	    ),
	    TP_fast_assign(HW_ASSIGN;
			   HWQ_ASSIGN;
			   TXQ_ASSIGN;
			   __entry->sn = sn;
			   __entry->sent = sent;
			   __entry->acked = acked;),
	    TP_printk(HW_PR_FMT HWQ_PR_FMT TXQ_PR_FMT " sn:%d sent:%d acked:%d",
		      HW_PR_ARG, HWQ_PR_ARG, TXQ_PR_ARG, __entry->sn, __entry->sent, __entry->acked)
);

TRACE_EVENT(scm80211_tx_result_ampdu,
	    TP_PROTO(struct ieee80211_hw *hw, struct scm80211_hwq *hwq,
		     struct scm80211_txq *txq, int ssn,
		     unsigned long *sent, unsigned long *ba),
	    TP_ARGS(hw, hwq, txq, ssn, sent, ba),
	    TP_STRUCT__entry(HW_ENTRY HWQ_ENTRY TXQ_ENTRY __field(int, ssn)
			     __field(u64, sent)
			     __field(u64, ba)
	    ),
	    TP_fast_assign(HW_ASSIGN;
			   HWQ_ASSIGN;
			   TXQ_ASSIGN;
			   __entry->ssn = ssn;
			   __entry->sent = get_unaligned_le64(sent);
			   __entry->ba = get_unaligned_le64(ba);),
	    TP_printk(HW_PR_FMT HWQ_PR_FMT TXQ_PR_FMT " ssn:%d sent:%016llx acked:%016llx",
		      HW_PR_ARG, HWQ_PR_ARG, TXQ_PR_ARG, __entry->ssn, __entry->sent, __entry->ba)
);

TRACE_EVENT(scm80211_start_txop,
	    TP_PROTO(struct ieee80211_hw *hw, struct txop *txop,
		     struct scm80211_txq *txq, int aifs, int bcnt, int n_mpdu, bool ampdu),
	    TP_ARGS(hw, txop, txq, aifs, bcnt, n_mpdu, ampdu),
	    TP_STRUCT__entry(HW_ENTRY TXOP_ENTRY TXQ_ENTRY __field(int, aifs)
			     __field(int, bcnt)
			     __field(int, n_mpdu)
			     __field(bool, ampdu)
			     __field(u8, fmt)
			     __field(u8, mcs)
			     __field(u8, nss)
			     __field(u8, fec)
			     __field(u8, chbw)
	    ),
	    TP_fast_assign(HW_ASSIGN;
			   TXOP_ASSIGN;
			   TXQ_ASSIGN;
			   __entry->aifs = aifs;
			   __entry->bcnt = bcnt;
			   __entry->n_mpdu = n_mpdu;
			   __entry->ampdu = ampdu;
			   __entry->fmt = txop->tv.FORMAT;
			   __entry->mcs = txop->tv.MCS;
			   __entry->nss = txop->tv.NUM_STS;
			   __entry->fec = txop->tv.FEC_CODING;
			   __entry->chbw = txop->tv.CH_BANDWIDTH;),
	    TP_printk(HW_PR_FMT TXOP_PR_FMT TXQ_PR_FMT
		      " aifs:%d bcnt:%d mpdu:%d ampdu:%d tv:<fmt:%d mcs:%d nss:%d fec:%d chbw:%d>",
		      HW_PR_ARG, TXOP_PR_ARG, TXQ_PR_ARG,
		      __entry->aifs, __entry->bcnt, __entry->n_mpdu, __entry->ampdu,
		      __entry->fmt, __entry->mcs, __entry->nss, __entry->fec, __entry->chbw)
);

TRACE_EVENT(scm80211_tx_return,
	    TP_PROTO(struct scm80211_hwq *hwq, const char *func),
	    TP_ARGS(hwq, func), TP_STRUCT__entry(HWQ_ENTRY __array(char, func, 40)
	    ),
	    TP_fast_assign(HWQ_ASSIGN;
			   strcpy(__entry->func, func);),
	    TP_printk(HWQ_PR_FMT " %s", HWQ_PR_ARG, __entry->func)
);

/* RX */
#define hwpriv(hw)		((struct ieee80211_hw_priv *) (hw->priv))

#define RXRING_ENTRY								\
	__field(u16, bptr)							\
	__field(u16, rptr)							\
	__field(u16, wptr)
#define RXRING_ASSIGN								\
	__entry->bptr = bptr,							\
	__entry->rptr = rptr;							\
	__entry->wptr = wptr;
#define RXRING_PR_FMT	" rx: bptr %04x rptr %04x wptr %04x"
#define RXRING_PR_ARG	__entry->bptr, __entry->rptr, __entry->wptr

DECLARE_EVENT_CLASS(scm80211_rx,
		    TP_PROTO(struct ieee80211_hw *hw, u16 bptr, u16 rptr, u16 wptr),
		    TP_ARGS(hw, bptr, rptr, wptr),
		    TP_STRUCT__entry(HW_ENTRY
				     RXRING_ENTRY),
		    TP_fast_assign(HW_ASSIGN;
				   RXRING_ASSIGN;),
		    TP_printk(HW_PR_FMT RXRING_PR_FMT, HW_PR_ARG, RXRING_PR_ARG)
);

DEFINE_EVENT(scm80211_rx, scm80211_rx_irq,
	     TP_PROTO(struct ieee80211_hw *hw, u16 bptr, u16 rptr, u16 wptr),
	     TP_ARGS(hw, bptr, rptr, wptr)
);

DEFINE_EVENT(scm80211_rx, scm80211_rx_poll,
	     TP_PROTO(struct ieee80211_hw *hw, u16 bptr, u16 rptr, u16 wptr),
	     TP_ARGS(hw, bptr, rptr, wptr)
);

/* phy txtime*/

DECLARE_EVENT_CLASS(scm80211_ppdu_txtime,
		    TP_PROTO(struct ieee80211_hw *hw, struct txvector *tv),
		    TP_ARGS(hw, tv), TP_STRUCT__entry(HW_ENTRY __field(int, fmt)
						      __field(int, rate)
		    ),
		    TP_fast_assign(HW_ASSIGN;
				   __entry->fmt = tv->FORMAT;
				   __entry->rate = tv->MCS;),
		    TP_printk(HW_PR_FMT " fmt:%d mcs:%d", HW_PR_ARG, __entry->fmt, __entry->rate)
);

DEFINE_EVENT(scm80211_ppdu_txtime, scm80211_ppdu_hw_cal_txtime,
	     TP_PROTO(struct ieee80211_hw *hw, struct txvector *tv), TP_ARGS(hw, tv)
);

DEFINE_EVENT(scm80211_ppdu_txtime, scm80211_ppdu_sw_cal_txtime,
	     TP_PROTO(struct ieee80211_hw *hw, struct txvector *tv), TP_ARGS(hw, tv)
);

TRACE_EVENT(scm80211_ppdu_cal_return_int,
	    TP_PROTO(struct ieee80211_hw *hw, u32 txtime),
	    TP_ARGS(hw, txtime), TP_STRUCT__entry(HW_ENTRY __field(int, txtime)
	    ),
	    TP_fast_assign(HW_ASSIGN;
			   __entry->txtime = txtime;),
	    TP_printk(HW_PR_FMT " ppdu txtime %u us", HW_PR_ARG, __entry->txtime)
);

TRACE_EVENT(scm80211_sta_rate_tbl_update,
	    TP_PROTO(struct ieee80211_hw *hw, struct ieee80211_vif *vif,
		     struct ieee80211_sta_rates *rates),
	    TP_ARGS(hw, vif, rates),
	    TP_STRUCT__entry(HW_ENTRY
			     VIF_ENTRY
			     RATES_ENTRY),
	    TP_fast_assign(HW_ASSIGN;
			   VIF_ASSIGN;
			   RATES_ASSIGN),
	    TP_printk(HW_PR_FMT VIF_PR_FMT RATES_PR_FMT, HW_PR_ARG, VIF_PR_ARG, RATES_PR_ARG)
);

/* DBG */
#define MAX_MSG_LEN	120

DECLARE_EVENT_CLASS(scm80211_msg_event,
		    TP_PROTO(struct va_format *vaf),
		    TP_ARGS(vaf), TP_STRUCT__entry(__dynamic_array(char, msg, MAX_MSG_LEN)
		    ),
		    TP_fast_assign(WARN_ON_ONCE(vsnprintf(__get_dynamic_array(msg),
							  MAX_MSG_LEN, vaf->fmt,
							  *vaf->va) >= MAX_MSG_LEN);),
		    TP_printk("%s", __get_str(msg))
);

DEFINE_EVENT(scm80211_msg_event, scm80211_info, TP_PROTO(struct va_format *vaf), TP_ARGS(vaf));
DEFINE_EVENT(scm80211_msg_event, scm80211_dbg, TP_PROTO(struct va_format *vaf), TP_ARGS(vaf));
DEFINE_EVENT(scm80211_msg_event, scm80211_err, TP_PROTO(struct va_format *vaf), TP_ARGS(vaf));

#endif /* !__MAC80211_DRIVER_TRACE || TRACE_HEADER_MULTI_READ */

#undef TRACE_INCLUDE_PATH
#define TRACE_INCLUDE_PATH .
#undef TRACE_INCLUDE_FILE
#define TRACE_INCLUDE_FILE trace
#include <trace/define_trace.h>
