/* SPDX-License-Identifier: GPL-2.0 */
/*
 *
 * Includes for cdc-acm.c
 *
 * Mainly take from usbnet's cdc-ether part
 *
 */

/*
 * CMSPAR, some architectures can't have space and mark parity.
 */

/*
 * The only reason to have several buffers is to accommodate assumptions
 * in line disciplines. They ask for empty space amount, receive our URB size,
 * and proceed to issue several 1-character writes, assuming they will fit.
 * The very first write takes a complete URB. Fortunately, this only happens
 * when processing onlcr, so we only need 2 buffers. These values must be
 * powers of 2.
 */
#define ACM_NW  16
#define ACM_NR  16

struct scm2020_usb_buff {
	u8			*buf;
	dma_addr_t		dmah;
	unsigned int		len;
	struct urb		*urb;
	struct list_head	list;
	int			idx;
	struct scm2020_usb_dl	*instance;
};

#define ABOOT_COMMAND_SZ 128
#define MAX_FW_DL_SIZE  0x10000
enum scm2020_dl_step {
	DL_INIT         = 0,
	DL_START        = 1,
	DL_STATUS       = 2,
	DL_FW           = 3,
	DL_PROG         = 4,
	DL_REBOOT       = 5,
	DL_FINISH       = 6,
};

enum frame_type {
	SMUX_FRAME_TYPE_STDIO       = (0x00),
	SMUX_FRAME_TYPE_HELLO       = (0x01),
	SMUX_FRAME_TYPE_HELLO_REPLY = (0x02),
	SMUX_FRAME_TYPE_ABOOT_CMD   = (0x03),
	SMUX_FRAME_TYPE_ABOOT_DATA  = (0x04),
	SMUX_FRAME_TYPE_HEART_BEAT  = (0x05),
	SMUX_ESC_CHAR               = (0x7D),
	SMUX_FRAME_DELIMITER        = (0x7E)
};

struct scm2020_msg {
	char resp[ABOOT_COMMAND_SZ + 1];
	enum frame_type type;
	unsigned int len;
	struct list_head list;
};

enum line_state {
	WAIT_FRAMESTART = 0,
	IN_FRAME,
	IN_ESCAPE
};

struct scm2020_usb_dl {
	struct usb_device	*udev;			/* the corresponding usb device */
	struct usb_interface	*intf;			/* interface */
	struct usb_interface	*control;		/* control interface */
	struct usb_interface	*data;			/* data interface */
	unsigned int in, out;				/* i/o pipes */

	struct urb		*ctrlurb;		/* urbs */
	u8			*ctrl_buffer;		/* buffers of urbs */
	dma_addr_t		ctrl_dma;		/* dma handles of buffers */
	unsigned int		ctrlsize;		/* buffer sizes for freeing */

	struct scm2020_usb_buff	wb[ACM_NW];
	struct list_head	write_list;
	spinlock_t		wb_lock;
	unsigned int		writesize;              /* max packet size for the output bulk ep */

	bool			device_lost;
	struct scm2020_usb_buff rb[ACM_NW];
	struct list_head        read_list;
	int			cmd_idx;
	struct scm2020_msg	cmd[ACM_NW];
	struct list_head        cmd_list;
	spinlock_t              rb_lock;
	unsigned int		readsize;		/* buffer sizes for freeing */
	char			response[ABOOT_COMMAND_SZ + 1];
	size_t			resp_len;

	struct workqueue_struct *workqueue;
	struct delayed_work     work;
	const struct firmware	*fw;
	size_t			off;
	enum frame_type		wtype;
	enum frame_type		rtype;

	enum scm2020_dl_step	step;
	spinlock_t              lock;
	struct mutex            mutex;
};

/**
 * Aboot preamble type definition
 */
#define ABOOT_PREAMBLE_SIZE     4	/* preamble char numbers */
#define ABOOT_PREAMBLE_UUUU     "UUUU"	/* aboot standard */
#define ABOOT_PREAMBLE_UABT     "UABT"	/* aboot tiny */
#define ABOOT_FRAME_DELIMITER   (0x7E)	/* frame delimiter */
