/* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause */
/*
 * Copyright (c) 2020-2023 Senscomm Semiconductor Co., Ltd.
 *
 */
#ifndef __SCM80211_UTIL_H__
#define __SCM80211_UTIL_H__

#include <linux/ieee80211.h>
#include <net/cfg80211.h>
#include <net/mac80211.h>
#include <linux/version.h>

#include "core.h"

#define IEEE80211_RTS_LEN   (sizeof(struct ieee80211_rts) + FCS_LEN)
#define IEEE80211_CTS_LEN   (sizeof(struct ieee80211_cts) + FCS_LEN)
#define IEEE80211_ACK_LEN   IEEE80211_CTS_LEN
/**
 * <FC>|<Dur>|<RA>|<TA>|<BA-C>|<C-BA>|<FCS>
 *  2     2    6    6     2      10     4
 */
#define IEEE80211_BACK_LEN	32	/* Compressed BA */

#define IEEE80211_MAX_HDR_LEN	30
#define IEEE80211_HTC_SIZE	4

#define IEEE80211_RX_HEADROOM					\
	ALIGN((sizeof(struct ieee80211_radiotap_he) +		\
	       sizeof(struct ieee80211_radiotap_he_mu)), 32)

#define MPDU_DELIMITER_SIZE	4

#define WLAN_EID_EXT_SPATIAL_REUSE  39

#define IEEE80211_DEF_NON_SRG_OBSS_PD_MAX_VALUE_M62			0xCE	/* -62 */
#define IEEE80211_DEF_NON_SRG_OBSS_PD_MAX_VALUE_M82			0xAE	/* -82 */
#define IEEE80211_DEF_NON_SRG_OBSS_PD_MIN_VALUE				0xAE	/* -82 */
#define IEEE80211_DEF_SRG_OBSS_PD_MIN_MAX_VALUE				0xAE	/* -82 */
#define IEEE80211_SR_NON_SRG_OBSS_PD_SR_DISALLOWED_PRESENT	0x02
#define IEEE80211_SR_NON_SRG_OFFSET_PRESENT					0x04
#define IEEE80211_SR_SRG_INFORMATION_PRESENT				0x08
#define IEEE80211_HESIGA_SR_VAL15_ALLOWED_PRESENT			0x10

#define IEEE80211_HE_OPERATION_RTS_THRESHOLD_MASK		0x00003ff0
#ifndef IEEE80211_AMPDU_TX_START_DELAY_ADDBA
#define IEEE80211_AMPDU_TX_START_DELAY_ADDBA			0
#endif

/**
 * VHT/HE aPPDUMaxTime is 5.484ms
 * HT aPPDUMaxTime is 10ms
 * use the shorter one
 **/
#define IEEE80211_PPDU_MAX_TXTIME	5484
#define IEEE80211_MAX_DURATION		0xFFFF
#define SCM80211_RTS_THRESHOLD		1440	/* rts threshold based on psdu txtime */
/**
 * Set THRESHOLD as IEEE80211_MAX_DURATION - 8TU
 * If txtime longer than IEEE80211_DURATION_THRESHOLD
 * enable protection
 **/
#define IEEE80211_DURATION_THRESHOLD	(IEEE80211_MAX_DURATION - 1024 * 8)

#define HE_VARIANT		(BIT(0) | BIT(1))
#define HE_LA_CTL		2
#define HE_CTLID_SHIFT		2
#define HE_CTLID_MASK		0x0000003C
#define HE_UNSOL_SHIFT		6
#define HE_MRQ_SHIFT		7
#define HE_NSS_SHIFT		8
#define HE_NSS_MASK		0x00000700
#define HE_MCS_SHIFT		11
#define HE_MCS_MASK		0x00007800
#define HE_DCM_SHIFT		15
#define HE_RU_ALLOC_SHIFT	16
#define HE_RU_ALLOC_MASK	0x00FF0000
#define HE_BW_SHIFT		24
#define HE_BW_MASK		0x03000000
#define HE_MSI_SHIFT		26
#define HE_MSI_MASK		0x1C000000
#define HE_TXB_SHIFT		29
#define HE_UL_TB_SHIFT		30
#define HE_LA_BOTH		3
#define VHT_VARIANT		BIT(0)
#define VHT_MRQ_SHIFT		2
#define VHT_MSI_SHIFT		3
#define VHT_MFSI_SHIFT		6
#define VHT_MFSI_MASK		0x000001C0
#define VHT_MFB_SHIFT		9
#define VHT_MFB_MASK		0x000FFE00
#define VHT_FB_TYPE_SHIFT	28
#define VHT_UNSOL_SHIFT		29
#define VHT_LA_BOTH		3

/* HTC & OMI */
#define	HTC_MASK(p1, p0)	((0xffffffff >> (31 - (p1) + (p0))) << (p0))
#define	HTC_GET_FLD(h, p1, p0)	(((h) & HTC_MASK(p1, p0)) >> (p0))
#define	HTC_SET_FLD(h, p1, p0)	(((h) << (p0)) & HTC_MASK(p1, p0))

#define	HTC_GET_TYPE(h)			HTC_GET_FLD(h, 1, 0)
#define	HTC_GET_CID(h)			HTC_GET_FLD(h, 5, 2)

#define	OMI_TYPE_CID			0x7	/* HTC type & CID */
#define	IS_OMI(h)			(HTC_GET_FLD(h, 5, 0) == OMI_TYPE_CID)

#define	OMI_GET_RX_NSS(h)		HTC_GET_FLD(h, 8, 6)
#define	OMI_GET_CHAN_WIDTH(h)		HTC_GET_FLD(h, 10, 9)
#define	OMI_GET_UL_MU_DIS(h)		HTC_GET_FLD(h, 11, 11)
#define	OMI_GET_TX_NSTS(h)		HTC_GET_FLD(h, 14, 12)
#define	OMI_GET_ER_SU_DIS(h)		HTC_GET_FLD(h, 15, 15)
#define	OMI_GET_DL_MU_RESND(h)		HTC_GET_FLD(h, 16, 16)
#define	OMI_GET_UL_MU_DATA_DIS(h)	HTC_GET_FLD(h, 17, 17)

#define	OMI_SET_RX_NSS(h)		HTC_SET_FLD(h, 8, 6)
#define	OMI_SET_CHAN_WIDTH(h)		HTC_SET_FLD(h, 10, 9)
#define	OMI_SET_UL_MU_DIS(h)		HTC_SET_FLD(h, 11, 11)
#define	OMI_SET_TX_NSTS(h)		HTC_SET_FLD(h, 14, 12)
#define	OMI_SET_ER_SU_DIS(h)		HTC_SET_FLD(h, 15, 15)
#define	OMI_SET_DL_MU_RESND(h)		HTC_SET_FLD(h, 16, 16)
#define	OMI_SET_UL_MU_DATA_DIS(h)	HTC_SET_FLD(h, 17, 17)

#define WLAN_CATEGORY_UNPROT_SIG	22
#define WLAN_EID_TWT			216
/* UNPROT_SIG action code */
enum ieee80211_unprot_sig_actioncode {
	WLAN_SIG_AID_SWITCH_REQ		= 0,
	WLAN_SIG_AID_SWITCH_RESP	= 1,
	WLAN_SIG_SYNC_CONTROL		= 2,
	WLAN_SIG_STA_INFO_ANNOUNCE	= 3,
	WLAN_SIG_EDCA_PARAM_SET		= 4,
	WLAN_SIG_EL_OPERATION		= 5,
	WLAN_SIG_TWT_SETUP		= 6,
	WLAN_SIG_TWT_TEARDOWN		= 7,
	WLAN_SIG_SECTOR_GID_LIST	= 8,
	WLAN_SIG_SECTOR_ID_FB		= 9,
	WLAN_SIG_TWT_INFO		= 11,
};

struct unprot_sig_action {
	u8 category;
	u8 action_code;
	u8 dialog_token;
	u8 variable[0];
} __packed;

/* TWT control */
#define TWT_NDP_PAGING_INDICATOR	0x01
#define TWT_RESPONDER_PM_MODE		0x02
#define TWT_NEGOTIATION_TYPE		0x0C
#define TWT_INFO_FRAME_DIS		0x10
#define TWT_WAKE_DURATION_UNIT		0x20

/* Individual TWT Requset type */
#define TWT_REQUEST			0x0001
#define TWT_SETUP_CMD			0x000E
#define TWT_TRIGGER			0x0010
#define TWT_IMPLICIT			0x0020
#define TWT_FLOW_TYPE			0x0040
#define TWT_FLOW_ID			0x0380
#define TWT_WAKE_INT_EXP		0x7C00
#define TWT_PROTECTION			0x8000

enum twt_negotype {
	INDIVIDUAL_TWT		= 0,
	INDIVIDUAL_TBTT		= 1,
	BROADCAST_TWT_GROUP	= 2,
	BROADCAST_TWT_SINGLE	= 3,
};

enum twt_setup_cmd {
	REQUEST_TWT		= 0,
	SUGGEST_TWT		= 1,
	DEMAND_TWT		= 2,
	TWT_GROUPING		= 3,
	ACCEPT_TWT		= 4,
	ALTERNATE_TWT		= 5,
	DICTATE_TWT		= 6,
	REJECT_TWT		= 7,
};

/* Parsed Information Elements */
struct ieee802_11_elems {
	const u8 *ie_start;
	size_t total_len;

	/* pointers to IEs */
	const struct ieee80211_he_operation *he_operation;
	const u8 *twt;
	u8 twt_len;
};

/* FIXME when it works on big endian platform */
#if KERNEL_VERSION(5, 16, 0) > LINUX_VERSION_CODE
#define le32_get_bits(val, mask)		FIELD_GET(mask, val)
#define le16_encode_bits(val, mask)		FIELD_PREP(mask, val)
#endif

#if KERNEL_VERSION(4, 20, 0) > LINUX_VERSION_CODE
#define dev_is_platform(dev)			((dev)->bus == &platform_bus_type)
#endif

#if KERNEL_VERSION(5, 0, 0) > LINUX_VERSION_CODE
#define DMA_MAPPING_ERROR			(~(dma_addr_t)0)
#endif

static inline bool ieee80211_iface_is_ap(enum nl80211_iftype type)
{
	return (type == NL80211_IFTYPE_AP ||
		type == NL80211_IFTYPE_AP_VLAN || type == NL80211_IFTYPE_P2P_GO);
}

static inline bool ieee80211_iface_is_monitor(enum nl80211_iftype type)
{
	return (type == NL80211_IFTYPE_MONITOR);
}

/**
 * NB: ieee80211_vif_to_wdev() can return NULL in the early stage of @vif
 */
static inline struct wiphy *ieee80211_vif_to_wiphy(struct ieee80211_vif *vif)
{
	struct wireless_dev *wdev = ieee80211_vif_to_wdev(vif);

	if (!wdev)
		return NULL;
	return wdev->wiphy;
}

#define ieee80211_tu_to_tsf(tu)	((tu) << 10)
#define ieee80211_tsf_to_tu(tsf)	((tsf) >> 10)

static inline struct ieee80211_hw *ieee80211_vif_to_hw(struct ieee80211_vif *vif)
{
	struct ieee80211_vif_priv *vif_priv = ieee80211_to_priv(vif);

	return vif_priv->hw;

	/* This does not work for monitor interface */
	// struct wireless_dev *wdev = ieee80211_vif_to_wdev(vif);

	// return wiphy_to_ieee80211_hw(wdev->wiphy);
}

static inline struct ieee80211_supported_band *ieee80211_get_tx_band(struct sk_buff *skb)
{
	struct ieee80211_tx_info *ti = IEEE80211_SKB_CB(skb);
	struct wiphy *wiphy = ieee80211_vif_to_wiphy(ti->control.vif);

	return wiphy->bands[ti->band];
}

static inline struct ieee80211_chanctx_conf __rcu *get_ieee80211_chanctx_conf(struct ieee80211_vif
									      *vif)
{
#if KERNEL_VERSION(5, 19, 0) > MAC80211_BASE
	return vif->chanctx_conf;
#else
	return vif->bss_conf.chanctx_conf;
#endif
}

#if KERNEL_VERSION(5, 19, 0) > MAC80211_BASE
#define rcu_dereference_chanctx_conf(x) rcu_dereference((x)->chanctx_conf)
#define rcu_access_pointer_chanctx_conf(x) rcu_access_pointer((x)->chanctx_conf)
#else
#define rcu_dereference_chanctx_conf(x) rcu_dereference((x)->bss_conf.chanctx_conf)
#define rcu_access_pointer_chanctx_conf(x) rcu_access_pointer((x)->bss_conf.chanctx_conf)

#endif

static inline bool get_ieee80211_mu_mimo_owner(struct ieee80211_vif *vif)
{
#if KERNEL_VERSION(5, 19, 0) > MAC80211_BASE
	return vif->mu_mimo_owner;
#else
	return vif->bss_conf.mu_mimo_owner;
#endif
}

static inline u32 get_ieee80211_supp_rates(struct ieee80211_sta *sta, enum nl80211_band band)
{
#if KERNEL_VERSION(5, 19, 0) > MAC80211_BASE
	return sta->supp_rates[band];
#else
	return sta->deflink.supp_rates[band];
#endif
}

static inline struct ieee80211_sta_ht_cap get_ieee80211_ht_cap(const struct ieee80211_sta *sta)
{
#if KERNEL_VERSION(5, 19, 0) > MAC80211_BASE
	return sta->ht_cap;
#else
	return sta->deflink.ht_cap;
#endif
}

static inline struct ieee80211_sta_vht_cap get_ieee80211_vht_cap(const struct ieee80211_sta *sta)
{
#if KERNEL_VERSION(5, 19, 0) > MAC80211_BASE
	return sta->vht_cap;
#else
	return sta->deflink.vht_cap;
#endif
}

static inline struct ieee80211_sta_he_cap get_ieee80211_he_cap(const struct ieee80211_sta *sta)
{
#if KERNEL_VERSION(5, 19, 0) > MAC80211_BASE
	return sta->he_cap;
#else
	return sta->deflink.he_cap;
#endif
}

static inline struct ieee80211_sta_txpwr get_ieee80211_txpwr(const struct ieee80211_sta *sta)
{
#if KERNEL_VERSION(5, 19, 0) > MAC80211_BASE
	return sta->txpwr;
#else
	return sta->deflink.txpwr;
#endif
}

static inline enum ieee80211_sta_rx_bandwidth get_ieee80211_rx_bandwidth(const struct ieee80211_sta
									 *sta)
{
#if KERNEL_VERSION(5, 19, 0) > MAC80211_BASE
	return sta->bandwidth;
#else
	return sta->deflink.bandwidth;
#endif
}

static inline u8 get_ieee80211_rx_nss(const struct ieee80211_sta *sta)
{
#if KERNEL_VERSION(5, 19, 0) > MAC80211_BASE
	return sta->rx_nss;
#else
	return sta->deflink.rx_nss;
#endif
}

static inline bool is_he_bss_color_enabled(struct ieee80211_vif *vif)
{
	bool bss_color_enabled;
#if KERNEL_VERSION(5, 10, 0) <= MAC80211_BASE
	bss_color_enabled = vif->bss_conf.he_bss_color.enabled;
#elif KERNEL_VERSION(5, 7, 0) <= MAC80211_BASE
	bss_color_enabled = !vif->bss_conf.he_bss_color.disabled;
#else
	struct ieee80211_he_operation *he_oper = &vif->bss_conf.he_operation;

	bss_color_enabled = !!(he_oper->he_oper_params & IEEE80211_HE_OPERATION_PARTIAL_BSS_COLOR);
#endif
	return bss_color_enabled;
}

#if KERNEL_VERSION(4, 20, 0) > MAC80211_BASE
#define dev_is_platform(dev) ((dev)->bus == &platform_bus_type)
#endif

static inline __le32 ieee80211_he_oper_params(struct ieee80211_bss_conf *conf)
{
	__le32 params;
#if KERNEL_VERSION(5, 7, 0) > MAC80211_BASE
	struct ieee80211_he_operation *he_oper = &conf->he_operation;

	params = he_oper->he_oper_params;
#else
	params = conf->he_oper.params;
#endif
	return params;
}

static inline u8 ieee80211_he_bss_color(struct ieee80211_bss_conf *conf)
{
#if KERNEL_VERSION(5, 7, 0) > MAC80211_BASE
	return conf->bss_color;
#else
	return conf->he_bss_color.color;
#endif
}

static inline bool ieee80211_he_partial_bss_color(struct ieee80211_bss_conf *conf)
{
#if KERNEL_VERSION(5, 7, 0) > MAC80211_BASE
	struct ieee80211_he_operation *he_oper = &conf->he_operation;

	return !!(he_oper->he_oper_params & IEEE80211_HE_OPERATION_BSS_COLOR_DISABLED);
#else
	return conf->he_bss_color.partial;
#endif
}

static inline bool scm80211_csa_is_finished(struct ieee80211_vif *vif)
{
#if KERNEL_VERSION(5, 19, 0) > MAC80211_BASE
	if (!vif || !vif->csa_active)
#else
	if (!vif || !vif->bss_conf.csa_active)
#endif

		return false;
#if KERNEL_VERSION(5, 10, 0) > MAC80211_BASE
	return ieee80211_csa_is_complete(vif);
#else
	return ieee80211_beacon_cntdwn_is_complete(vif);
#endif
}

static inline bool ieee80211_is_he_sta(struct ieee80211_sta *sta)
{
#if KERNEL_VERSION(5, 19, 0) > MAC80211_BASE
	return sta && sta->he_cap.has_he;
#else
	return sta && sta->deflink.he_cap.has_he;
#endif
}

static inline bool ieee80211_is_ht_sta(struct ieee80211_sta *sta)
{
#if KERNEL_VERSION(5, 19, 0) > MAC80211_BASE
	return sta && sta->ht_cap.ht_supported;
#else
	return sta && sta->deflink.ht_cap.ht_supported;
#endif
}

static inline bool ieee80211_is_vht_sta(struct ieee80211_sta *sta)
{
#if KERNEL_VERSION(5, 19, 0) > MAC80211_BASE
	return sta && sta->vht_cap.vht_supported;
#else
	return sta && sta->deflink.vht_cap.vht_supported;
#endif
}

static inline bool ieee80211_is_bar(__le16 fc)
{
	return ieee80211_is_back_req(fc);
}

static inline bool ieee80211_is_action_no_ack(__le16 fc)
{
	u16 ftype;

	ftype = fc & cpu_to_le16(IEEE80211_FCTL_FTYPE | IEEE80211_FCTL_STYPE);
	return ftype == cpu_to_le16(IEEE80211_FTYPE_MGMT | (IEEE80211_STYPE_ACTION + 1));
}

/**
 * ieee80211_solicts_ack() - check if a frame solicits an immediate response
 */
static inline bool ieee80211_solicits_ack(struct ieee80211_hdr *hdr)
{
	__le16 fc = hdr->frame_control;

	if (is_multicast_ether_addr(hdr->addr1))
		return false;

	if (ieee80211_is_data_qos(fc) || ieee80211_is_qos_nullfunc(fc)) {
		u8 qos_ack_policy;

		qos_ack_policy = *ieee80211_get_qos_ctl(hdr);
		qos_ack_policy &= IEEE80211_QOS_CTL_ACK_POLICY_MASK;
		switch (qos_ack_policy) {
		case IEEE80211_QOS_CTL_ACK_POLICY_NORMAL:
			return true;
		case IEEE80211_QOS_CTL_ACK_POLICY_NOACK:
		case IEEE80211_QOS_CTL_ACK_POLICY_NO_EXPL:
		case IEEE80211_QOS_CTL_ACK_POLICY_BLOCKACK:
			return false;
		}
	} else if (ieee80211_is_mgmt(fc)) {
		return !ieee80211_is_action_no_ack(fc);
	} else if (ieee80211_is_ctl(fc)) {
		return (ieee80211_is_back_req(fc) || ieee80211_is_pspoll(fc));
	}
	return true;
}

/**
 * DOC: Transmission rate/mcs
 *
 * The following data structures are used to represent PPDU parameter (which
 * will eventually encoded to TXVECTOR):
 *
 * struct ieee80211_rate (cfg80211.h)
 * - Non-HT rate (i.e., 11a/b/g) information
 * - @wiphy->bands[band].bitrates[index]
 *
 * struct rate_info (cfg80211.h)
 * - Used to calculate the frame bitrate
 *
 * struct ieee80211_tx_rate (mac80211.h)
 * - Internally, we use IEEE80211_TX_RC_GREEN_FIELD as IEEE80211_TX_RC_HE_MCS
 *   for we do not support HT greenfield format.
 *
 * struct ieee80211_sta_rates (mac80211.h)
 * - The driver does not access to these objects directly.
 *
 * Since mac80211 rate controller does not yet handle HE format, the driver
 * internally convert struct ieee80211_tx_rate (embedded in
 * &struct ieee80211_tx_info) into struct rate_info.
 */

/* Reuse the unused flags for he rates */
#define IEEE80211_TX_RC_HE_MCS	IEEE80211_TX_RC_GREEN_FIELD
#define IEEE80211_HE_GI_0u8     0
#define IEEE80211_HE_GI_1u6     IEEE80211_TX_RC_SHORT_GI
#define IEEE80211_HE_GI_3u2     IEEE80211_TX_RC_USE_SHORT_PREAMBLE

static inline enum cbw ieee80211_ppdu_bandwidth(struct ieee80211_tx_rate *r)
{
	if (r->flags & IEEE80211_TX_RC_160_MHZ_WIDTH)
		return CBW160;
	if (r->flags & IEEE80211_TX_RC_80_MHZ_WIDTH)
		return CBW80;
	if (r->flags & IEEE80211_TX_RC_40_MHZ_WIDTH)
		return CBW40;
	return CBW20;
}

static inline int ieee80211_rate_get_nss(struct ieee80211_tx_rate *r)
{
	if (r->flags & (IEEE80211_TX_RC_VHT_MCS | IEEE80211_TX_RC_HE_MCS))
		return ieee80211_rate_get_vht_nss(r);
	if (r->flags & IEEE80211_TX_RC_MCS)
		return (r->idx >> 3) + 1;
	else
		return 1;
}

static inline int ieee80211_rate_get_mcs(struct ieee80211_tx_rate *r)
{
	if (r->flags & (IEEE80211_TX_RC_VHT_MCS | IEEE80211_TX_RC_HE_MCS))
		return ieee80211_rate_get_vht_mcs(r);
	if (r->flags & IEEE80211_TX_RC_MCS)
		return (r->idx & 0x07);
	else
		return r->idx;
}

static inline void ieee80211_rate_set(struct ieee80211_tx_rate *r, u8 mcs, u8 nss)
{
	if (r->flags & IEEE80211_TX_RC_MCS)
		r->idx = ((nss - 1) << 3) | mcs;
	if (r->flags & (IEEE80211_TX_RC_VHT_MCS | IEEE80211_TX_RC_HE_MCS))
		ieee80211_rate_set_vht(r, mcs, nss);
}

static inline bool ieee80211_rate_is_non_ht(struct ieee80211_tx_rate *r)
{
	return ((r->flags & IEEE80211_TX_RC_MCS) == 0 &&
		(r->flags & IEEE80211_TX_RC_VHT_MCS) == 0 &&
		(r->flags & IEEE80211_TX_RC_HE_MCS) == 0);
}

static inline bool ieee80211_rate_is_ht(struct ieee80211_tx_rate *r)
{
	return (r->flags & IEEE80211_TX_RC_MCS);
}

static inline bool ieee80211_rate_is_vht(struct ieee80211_tx_rate *r)
{
	return (r->flags & IEEE80211_TX_RC_VHT_MCS);
}

static inline bool ieee80211_rate_is_he(struct ieee80211_tx_rate *r)
{
	return (r->flags & IEEE80211_TX_RC_HE_MCS);
}

/**
 * ieee80211_mpdu_len() - return the length of an mpdu
 *
 * @skb: mpdu
 *
 * Returns: the length of IEEE802.11 MPDU that includes the
 * MAC header through FCS.
 *
 * Note:
 * This function accesses ti field of struct scm80211_tx_info,
 * not directly accesses skb->cb for that information is
 * already corrupted.
 */
static inline size_t ieee80211_mpdu_len(struct sk_buff *skb)
{
	struct ieee80211_tx_info *ti = IEEE80211_SKB_CB(skb);
	struct ieee80211_key_conf *key = ti->control.hw_key;
	size_t len;

	len = skb->len;
	len += key ? key->icv_len : 0;
	if (key && key->cipher == WLAN_CIPHER_SUITE_TKIP && key->icv_len == IEEE80211_TKIP_ICV_LEN)
		len += 8;	/* Michael MIC size */

	len += FCS_LEN;

	return len;
}

/**
 * ieee80211_subframe_len() - subframe length
 */
static inline size_t ieee80211_subframe_len(struct sk_buff *skb)
{
	size_t len = ieee80211_mpdu_len(skb) + MPDU_DELIMITER_SIZE;

	return roundup(len, 4);
}

static inline int ieee80211_sifs(enum nl80211_band band)
{
	return (band == NL80211_BAND_2GHZ) ? 10 : 16;
}

struct ieee80211_rate *ieee80211_compute_ack_rate(struct ieee80211_hw *hw,
						  struct ieee80211_vif *vif,
						  struct ieee80211_tx_info *ti);

u16 ieee80211_compute_ack_time(struct ieee80211_hw *hw,
			       struct ieee80211_vif *vif,
			       struct ieee80211_tx_info *ti, size_t ack_len);

u16 ieee80211_compute_rts_time(struct ieee80211_hw *hw,
			       struct ieee80211_vif *vif, struct ieee80211_tx_info *ti);

int ieee80211_max_ampdu_len(struct ieee80211_sta *sta, struct ieee80211_tx_rate *rate);

u16 ieee80211_vht_partial_aid(u16 aid, const u8 *addr, int group);

u8 *ieee80211_get_bssid(struct ieee80211_hdr *hdr, size_t len, enum nl80211_iftype type);

int ieee80211_chandef_create(struct cfg80211_chan_def *chandef,
			     const struct ieee80211_sta *ap,
			     const struct ieee80211_ht_operation *ht_oper,
			     const struct ieee80211_vht_operation *vht_oper);

int ieee80211_link_adap_to_rate(struct ieee80211_hw *hw, __le32 htc,
				struct ieee80211_tx_rate *rate, bool *stbc);

/* This works only for skb that does not include txvector */
static inline u16 ieee80211_get_sn(struct sk_buff *skb)
{
	struct ieee80211_hdr *hdr = (struct ieee80211_hdr *)skb->data;

	return IEEE80211_SEQ_TO_SN(le16_to_cpu(hdr->seq_ctrl));
}

bool scm80211_rx_parse_needed(const u8 *tx_bssid, struct ieee80211_sta *sta,
			      struct ieee80211_bss_conf *info);

void scm80211_parse_elems(const u8 *start, size_t len,
			  bool action, struct ieee802_11_elems *elems);

static inline int scm80211_get_skb_fragments(struct sk_buff *skb)
{
	int n = 1 + skb_shinfo(skb)->nr_frags;
	struct sk_buff *skb_iter;

	skb_walk_frags(skb, skb_iter)
	    n += 1 + skb_shinfo(skb_iter)->nr_frags;

	return n;
}

static inline bool scm80211_concurrent_mode(struct ieee80211_hw *hw)
{
	struct ieee80211_hw_priv *priv = hw->priv;

	return (bitmap_weight(&priv->vif_map, SCM80211_NUM_VIF) > 1);
}

static inline bool ieee80211_maybe_offchannel_frame(__le16 fc)
{
	return (ieee80211_is_probe_req(fc) || ieee80211_is_action(fc));
}

void scm80211_update_sr(struct ieee80211_hw *hw, struct ieee80211_vif *vif);

void scm80211_init_sr(struct ieee80211_hw *hw, struct ieee80211_vif *vif);

void ieee80211_parse_he_sr(struct ieee80211_hw *hw, struct ieee80211_vif *vif, const u8 *ie);

void ieee80211_parse_he_operation(struct ieee80211_hw *hw, struct ieee80211_vif *vif,
				  const struct ieee80211_he_operation *ie,
				  struct ieee80211_sta *sta);

bool ieee80211_is_highest_mcs(struct ieee80211_sta *sta, struct ieee80211_tx_rate *r);

#define bitsizeof(x) (sizeof(x) * BITS_PER_BYTE)

static inline char *ieee80211_vif_name(struct ieee80211_vif *vif)
{
	struct wireless_dev *wdev = ieee80211_vif_to_wdev(vif);

	if (wdev && wdev->netdev)
		return wdev->netdev->name;
	else
		return NULL;
}

bool scm80211_is_radar_freq(u16 center_freq);

int scm80211_process_omi(struct ieee80211_hw *hw, struct ieee80211_vif *vif,
			 struct ieee80211_sta *sta, u32 htc, bool is_rx);

bool scm80211_dbg_construct_rate(struct ieee80211_sta *sta,
					enum ppdu_format fmt, u8 rate_idx,
					enum cbw bw, u8 nss,
					enum gi_type gtype, enum ltf_type ltf_type, bool is_ldpc);

extern const char *ieee80211_iftype_str[];
extern const char *ieee80211_sta_state_str[];
extern const char *cipher_suite_str[];
extern const char *const chanwidth_str[];

enum HEX_STR_POS {
	POS_10 = 10,
	POS_16 = 16,
	POS_NONE,
};

int string_split(char *spl[], char *str, char *delim, int n);
int decstr2int(char *str, enum HEX_STR_POS pos);
void uppercase(char *u_str, const char *str);

#endif /* __SCM80211_UTIL_H__ */
