// SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause
/*
 * Copyright (c) 2020-2023 Senscomm Semiconductor Co., Ltd.
 */
#define func_fmt(str)           "[PRIV] "str
#define func_trace		PRIV
#include "core.h"

#define SENSCOMM_OUI	0x26205A
enum scm80211_vendor_cmd {
	SCM80211_VENDOR_CMD_TEST			= 0x01,
	SCM80211_VENDOR_CMD_SET_PRIVATE_WPS_P2P_IE	= 0x02,
};

/**
 * scm80211_vendor_cmd_test() - test vendor cmd handler
 *
 * This function can be called in by using
 * wpa_cli cmd: vendor 0x26205A(OUI) 1(subcmd) [hex formatted data]
 *
 * e.g > vendor 0x26205A 1 41424344454647  '0x41'is 'A' in ASCII
 * it will receive 'ABCDEFG'
 */
static int scm80211_vendor_cmd_test(struct wiphy *wiphy, struct wireless_dev *wdev,
				    const void *data, int data_len)
{
	struct ieee80211_hw *hw = wiphy_to_ieee80211_hw(wiphy);
	struct ieee80211_vif *vif = wdev_to_ieee80211_vif(wdev);
	char test[16 + 1] = "\0";

	data_len = min(data_len, 16);
	memcpy(&test, data, data_len);
	test[data_len] = '\0';

	scm80211_trace(hw, "%s test cmd %s\n", __func__, test);

	(void)hw;
	(void)vif;

	return 0;
}

static const struct wiphy_vendor_command scm80211_vendor_commands[] = {
	{
	 .info = {
		  .vendor_id = SENSCOMM_OUI,
		  .subcmd = SCM80211_VENDOR_CMD_TEST,
		  },
	 .doit = scm80211_vendor_cmd_test,
	 .flags = WIPHY_VENDOR_CMD_NEED_NETDEV | WIPHY_VENDOR_CMD_NEED_RUNNING,
	 .policy = VENDOR_CMD_RAW_DATA,
	 .maxattr = 0,
	  },
};

/**
 * scm80211_register_vendor_cmds() - to register the vendor cmds
 */
void scm80211_register_vendor_cmds(struct ieee80211_hw *hw)
{
	hw->wiphy->vendor_commands = scm80211_vendor_commands;
	hw->wiphy->n_vendor_commands = ARRAY_SIZE(scm80211_vendor_commands);
}
