/* SPDX-License-Identifier: GPL-2.0 OR BSD-2-Clause */
/*
 * Copyright (c) 2020-2023 Senscomm Semiconductor Co., Ltd.
 *
 */
#ifndef __SCM80211_WOW_H__
#define __SCM80211_WOW_H__
#include <linux/types.h>
#ifdef SCM80211_WOW

#define SCM80211_WOW_BMAP_FILTER_MAX 4
#define SCM80211_WOW_BMAP_PATTERN_SIZE_MAX 128
#define SCM80211_WOW_BMAP_MASK_SIZE_MAX SCM80211_WOW_BMAP_PATTERN_SIZE_MAX

//wow
enum scm80211_wow_wake_reason {
	WOW_WAKEUP_R_UNSPECIFIED = 0,
	WOW_WAKEUP_R_AP_ASSOC_LOST,
	WOW_WAKEUP_R_LOW_RSSI,
	WOW_WAKEUP_R_DEAUTH_RECVD,
	WOW_WAKEUP_R_DISASSOC_RECVD,
	WOW_WAKEUP_R_GTK_HS_ERR,
	WOW_WAKEUP_R_EAP_REQ,
	WOW_WAKEUP_R_FOURWAY_HS_RECV,
	WOW_WAKEUP_R_TIMER_INTR_RECV,
	WOW_WAKEUP_R_PATTERN_MATCH_FOUND,
	WOW_WAKEUP_R_RECV_MAGIC_PACKET,
};

#ifndef _BIT
#define _BIT(nr)			((int)(1) << (nr))
#endif

enum scm80211_wow_wake_event {
	WOW_WAKEUP_E_MAGIC_PKT_RECVD_EVENT = _BIT(0),
	WOW_WAKEUP_E_GTK_ERR_EVENT = _BIT(1),
	WOW_WAKEUP_E_EAPOL_RECVD_EVENT = _BIT(2),
	WOW_WAKEUP_E_PATTERN_MATCH_EVENT = _BIT(3),
};

struct  scm80211_wow_bmap_filter_pattern {
	u32 pattern_offset;
	u32 pattern_len;
	u32 bitmask_len;

	u8 pattern[SCM80211_WOW_BMAP_PATTERN_SIZE_MAX];
	u8 bitmask[SCM80211_WOW_BMAP_MASK_SIZE_MAX];
};

struct scm80211_wow_bmap_filter {
	u8 valid;
	u8 index;
	struct scm80211_wow_bmap_filter_pattern bmap_filter_pattern;
};

/* definition for WoW global context */
struct scm80211_wow_context {
	u32 enabled;
	u32 event_flag;
	u32 nr_drop_frames;
	//
	u32 nr_bmap_filter;
	u32 bmap_filter_max;
	struct scm80211_wow_bmap_filter bmap_filter[SCM80211_WOW_BMAP_FILTER_MAX];
};

int scm80211_wow_set_events(struct ieee80211_hw *hw);
int scm80211_wow_attach(struct ieee80211_hw *hw);
int scm80211_wow_detach(struct ieee80211_hw *hw);

#endif //SCM80211_WOW
#endif /* __SCM80211_WOW_H__ */
